/*
 * source/data/EditXWindow.c
 * SpacePigs VectorMania
 * 
 * West window and gadgets
 */

#include "vector/vector.h"
#include "graphics/graphics.h"
#include "edit/edit.h"

extern struct Image ImageErase1, ImageErase2;
extern struct Image ImageLine1, ImageLine2;
extern struct Image ImageSurface1, ImageSurface2;
extern struct Image ImageVertex1, ImageVertex2;
extern struct Image ImageFlip1, ImageFlip2;
extern struct Image ImageZoomout1, ImageZoomout2;
extern struct Image ImageZoomin1, ImageZoomin2;
extern struct Image ImageRight1, ImageRight2;
extern struct Image ImageLeft1, ImageLeft2;
extern struct Image ImageDown1, ImageDown2;
extern struct Image ImageUp1, ImageUp2;
extern struct Image ImageGrab1, ImageGrab2;
extern struct Image ImageBall1, ImageBall2;

#define GADGET_USERDATA	NULL

static struct Gadget Gadget_Ball =
{
    NULL,			/* next gadget */
    -16, 61,			/* origin XY of hit box relative to window
				 * TopLeft */
    16, 9,			/* hit box width and height */
    GADGHIMAGE + GADGIMAGE + GRELRIGHT,	/* gadget flags */
    GADGIMMEDIATE + RIGHTBORDER,/* activation flags */
    BOOLGADGET + GZZGADGET,	/* gadget type flags */
    (APTR) & ImageBall1,	/* gadget border or image to be rendered */
    (APTR) & ImageBall2,	/* alternate imagery for selection */
    NULL,			/* first IntuiText structure */
    NULL,			/* gadget mutual-exclude long word */
    NULL,			/* SpecialInfo structure */
    GADGET_BALL,		/* user-definable data */
    GADGET_USERDATA		/* pointer to user-definable data */
};

static struct Gadget Gadget_Grab =
{
    &Gadget_Ball,		/* next gadget */
    -69, -10,			/* origin XY of hit box relative to window
				 * TopLeft */
    16, 11,			/* hit box width and height */
    GADGHIMAGE + GADGIMAGE + GRELBOTTOM + GRELRIGHT,	/* gadget flags */
    GADGIMMEDIATE + BOTTOMBORDER,	/* activation flags */
    BOOLGADGET + GZZGADGET,	/* gadget type flags */
    (APTR) & ImageGrab1,	/* gadget border or image to be rendered */
    (APTR) & ImageGrab2,	/* alternate imagery for selection */
    NULL,			/* first IntuiText structure */
    NULL,			/* gadget mutual-exclude long word */
    NULL,			/* SpecialInfo structure */
    GADGET_GRAB,		/* user-definable data */
    GADGET_USERDATA		/* pointer to user-definable data */
};

static struct Gadget Gadget_Erase =
{
    &Gadget_Grab,		/* next gadget */
    -16, 71,			/* origin XY of hit box relative to window
				 * TopLeft */
    16, 9,			/* hit box width and height */
    GADGHIMAGE + GADGIMAGE + GRELRIGHT,	/* gadget flags */
    RELVERIFY + RIGHTBORDER,	/* activation flags */
    BOOLGADGET + GZZGADGET,	/* gadget type flags */
    (APTR) & ImageErase1,	/* gadget border or image to be rendered */
    (APTR) & ImageErase2,	/* alternate imagery for selection */
    NULL,			/* first IntuiText structure */
    NULL,			/* gadget mutual-exclude long word */
    NULL,			/* SpecialInfo structure */
    GADGET_ERASE,		/* user-definable data */
    GADGET_USERDATA		/* pointer to user-definable data */
};

static struct Gadget Gadget_Line =
{
    &Gadget_Erase,		/* next gadget */
    -16, 51,			/* origin XY of hit box relative to window
				 * TopLeft */
    16, 9,			/* hit box width and height */
    GADGHIMAGE + GADGIMAGE + GRELRIGHT,	/* gadget flags */
    GADGIMMEDIATE + RIGHTBORDER,/* activation flags */
    BOOLGADGET + GZZGADGET,	/* gadget type flags */
    (APTR) & ImageLine1,	/* gadget border or image to be rendered */
    (APTR) & ImageLine2,	/* alternate imagery for selection */
    NULL,			/* first IntuiText structure */
    NULL,			/* gadget mutual-exclude long word */
    NULL,			/* SpecialInfo structure */
    GADGET_LINE,		/* user-definable data */
    GADGET_USERDATA		/* pointer to user-definable data */
};

static struct Gadget Gadget_Surface =
{
    &Gadget_Line,		/* next gadget */
    -16, 41,			/* origin XY of hit box relative to window
				 * TopLeft */
    16, 9,			/* hit box width and height */
    GADGHIMAGE + GADGIMAGE + GRELRIGHT,	/* gadget flags */
    GADGIMMEDIATE + RIGHTBORDER,/* activation flags */
    BOOLGADGET + GZZGADGET,	/* gadget type flags */
    (APTR) & ImageSurface1,	/* gadget border or image to be rendered */
    (APTR) & ImageSurface2,	/* alternate imagery for selection */
    NULL,			/* first IntuiText structure */
    NULL,			/* gadget mutual-exclude long word */
    NULL,			/* SpecialInfo structure */
    GADGET_SURFACE,		/* user-definable data */
    GADGET_USERDATA		/* pointer to user-definable data */
};

static struct Gadget Gadget_Vertex =
{
    &Gadget_Surface,		/* next gadget */
    -16, 31,			/* origin XY of hit box relative to window
				 * TopLeft */
    16, 9,			/* hit box width and height */
    GADGHIMAGE + GADGIMAGE + GRELRIGHT,	/* gadget flags */
    GADGIMMEDIATE + RIGHTBORDER,/* activation flags */
    BOOLGADGET + GZZGADGET,	/* gadget type flags */
    (APTR) & ImageVertex1,	/* gadget border or image to be rendered */
    (APTR) & ImageVertex2,	/* alternate imagery for selection */
    NULL,			/* first IntuiText structure */
    NULL,			/* gadget mutual-exclude long word */
    NULL,			/* SpecialInfo structure */
    GADGET_VERTEX,		/* user-definable data */
    GADGET_USERDATA		/* pointer to user-definable data */
};

static struct Gadget Gadget_Flip =
{
    &Gadget_Vertex,		/* next gadget */
    3, -10,			/* origin XY of hit box relative to window
				 * TopLeft */
    14, 11,			/* hit box width and height */
    GADGHIMAGE + GADGIMAGE + GRELBOTTOM,	/* gadget flags */
    RELVERIFY,			/* activation flags */
    BOOLGADGET + GZZGADGET,	/* gadget type flags */
    (APTR) & ImageFlip1,	/* gadget border or image to be rendered */
    (APTR) & ImageFlip2,	/* alternate imagery for selection */
    NULL,			/* first IntuiText structure */
    NULL,			/* gadget mutual-exclude long word */
    NULL,			/* SpecialInfo structure */
    GADGET_FLIP,		/* user-definable data */
    GADGET_USERDATA		/* pointer to user-definable data */
};

static struct Gadget Gadget_Zoomout =
{
    &Gadget_Flip,		/* next gadget */
    -16, 21,			/* origin XY of hit box relative to window
				 * TopLeft */
    16, 9,			/* hit box width and height */
    GADGHIMAGE + GADGIMAGE + GRELRIGHT,	/* gadget flags */
    RELVERIFY + RIGHTBORDER,	/* activation flags */
    BOOLGADGET + GZZGADGET,	/* gadget type flags */
    (APTR) & ImageZoomout1,	/* gadget border or image to be rendered */
    (APTR) & ImageZoomout2,	/* alternate imagery for selection */
    NULL,			/* first IntuiText structure */
    NULL,			/* gadget mutual-exclude long word */
    NULL,			/* SpecialInfo structure */
    GADGET_ZOOMOUT,		/* user-definable data */
    GADGET_USERDATA		/* pointer to user-definable data */
};

static struct Gadget Gadget_Zoomin =
{
    &Gadget_Zoomout,		/* next gadget */
    -16, 11,			/* origin XY of hit box relative to window
				 * TopLeft */
    16, 9,			/* hit box width and height */
    GADGHIMAGE + GADGIMAGE + GRELRIGHT,	/* gadget flags */
    RELVERIFY + RIGHTBORDER,	/* activation flags */
    BOOLGADGET + GZZGADGET,	/* gadget type flags */
    (APTR) & ImageZoomin1,	/* gadget border or image to be rendered */
    (APTR) & ImageZoomin2,	/* alternate imagery for selection */
    NULL,			/* first IntuiText structure */
    NULL,			/* gadget mutual-exclude long word */
    NULL,			/* SpecialInfo structure */
    GADGET_ZOOMIN,		/* user-definable data */
    GADGET_USERDATA		/* pointer to user-definable data */
};

static struct Gadget Gadget_Right =
{
    &Gadget_Zoomin,		/* next gadget */
    -33, -10,			/* origin XY of hit box relative to window
				 * TopLeft */
    14, 11,			/* hit box width and height */
    GADGHIMAGE + GADGIMAGE + GRELBOTTOM + GRELRIGHT,	/* gadget flags */
    RELVERIFY + GADGIMMEDIATE + BOTTOMBORDER,	/* activation flags */
    BOOLGADGET + GZZGADGET,	/* gadget type flags */
    (APTR) & ImageRight1,	/* gadget border or image to be rendered */
    (APTR) & ImageRight2,	/* alternate imagery for selection */
    NULL,			/* first IntuiText structure */
    NULL,			/* gadget mutual-exclude long word */
    NULL,			/* SpecialInfo structure */
    GADGET_RIGHT,		/* user-definable data */
    GADGET_USERDATA		/* pointer to user-definable data */
};

static struct Gadget Gadget_Left =
{
    &Gadget_Right,		/* next gadget */
    -51, -10,			/* origin XY of hit box relative to window
				 * TopLeft */
    16, 11,			/* hit box width and height */
    GADGHIMAGE + GADGIMAGE + GRELBOTTOM + GRELRIGHT,	/* gadget flags */
    RELVERIFY + GADGIMMEDIATE + BOTTOMBORDER,	/* activation flags */
    BOOLGADGET + GZZGADGET,	/* gadget type flags */
    (APTR) & ImageLeft1,	/* gadget border or image to be rendered */
    (APTR) & ImageLeft2,	/* alternate imagery for selection */
    NULL,			/* first IntuiText structure */
    NULL,			/* gadget mutual-exclude long word */
    NULL,			/* SpecialInfo structure */
    GADGET_LEFT,		/* user-definable data */
    GADGET_USERDATA		/* pointer to user-definable data */
};

static struct Gadget Gadget_Down =
{
    &Gadget_Left,		/* next gadget */
    -16, -18,			/* origin XY of hit box relative to window
				 * TopLeft */
    16, 9,			/* hit box width and height */
    GADGHIMAGE + GADGIMAGE + GRELBOTTOM + GRELRIGHT,	/* gadget flags */
    RELVERIFY + RIGHTBORDER,	/* activation flags */
    BOOLGADGET + GZZGADGET,	/* gadget type flags */
    (APTR) & ImageDown1,	/* gadget border or image to be rendered */
    (APTR) & ImageDown2,	/* alternate imagery for selection */
    NULL,			/* first IntuiText structure */
    NULL,			/* gadget mutual-exclude long word */
    NULL,			/* SpecialInfo structure */
    GADGET_DOWN,		/* user-definable data */
    GADGET_USERDATA		/* pointer to user-definable data */
};

static struct Gadget Gadget_Up =
{
    &Gadget_Down,		/* next gadget */
    -16, -28,			/* origin XY of hit box relative to window
				 * TopLeft */
    16, 9,			/* hit box width and height */
    GADGHIMAGE + GADGIMAGE + GRELBOTTOM + GRELRIGHT,	/* gadget flags */
    RELVERIFY + RIGHTBORDER,	/* activation flags */
    BOOLGADGET + GZZGADGET,	/* gadget type flags */
    (APTR) & ImageUp1,		/* gadget border or image to be rendered */
    (APTR) & ImageUp2,		/* alternate imagery for selection */
    NULL,			/* first IntuiText structure */
    NULL,			/* gadget mutual-exclude long word */
    NULL,			/* SpecialInfo structure */
    GADGET_UP,			/* user-definable data */
    GADGET_USERDATA		/* pointer to user-definable data */
};

#define GADGETLIST Gadget_Up

static struct NewWindow newwindow =
{
    440, 1,			/* window XY origin relative to TopLeft of
				 * screen */
    200, 125,			/* window width and height */
    0, 2,			/* detail and block pens */
    IDCMP_CLOSEWINDOW | IDCMP_NEWSIZE | IDCMP_MOUSEBUTTONS | IDCMP_GADGETDOWN | IDCMP_GADGETUP | IDCMP_MENUPICK | IDCMP_RAWKEY | IDCMP_ACTIVEWINDOW | IDCMP_INACTIVEWINDOW | IDCMP_MENUVERIFY | IDCMP_INTUITICKS,	/* IDCMP flags */
    WINDOWSIZING + WINDOWDRAG + WINDOWDEPTH + GIMMEZEROZERO + WINDOWCLOSE + ACTIVATE + NOCAREREFRESH,	/* other window flags */
    &GADGETLIST,		/* first gadget in gadget list */
    NULL,			/* custom CHECKMARK imagery */
    "West",			/* window title */
    NULL,			/* custom screen pointer */
    NULL,			/* custom bitmap */
    100, 50,			/* minimum width and height */
    1000, 1000,			/* maximum width and height */
    CUSTOMSCREEN		/* destination screen type */
};

WINDOWDATA editxwindow =
{
    WINDOW_X,			/* type */
    &newwindow,			/* newwindow */
    0,				/* flags */
    0,				/* window */
    0,				/* rastport */
    0, 0,			/* min,max x */
    0, 0,			/* min,max y */
    0, 0,			/* x,y center */
    0,				/* cursorbuffer */
    0, 0,			/* cursor x,y */
    0				/* Menu */
};
