/*
 * source/data/menu.c
 * SpacePigs VectorMania
 * 
 * Menu definition
 */

#include <exec/types.h>
#include <intuition/intuition.h>
#include <libraries/gadtools.h>

#include "other/debug.h"
#include "vector/vector.h"
#include "graphics/graphics.h"
#include "edit/edit.h"
#include "edit/edit_globals.h"

#define STDMENUFLAGS	ITEMTEXT
#define STDITEMFLAGS	ITEMTEXT
#define STDCHECK0FLAGS	STDITEMFLAGS|CHECKIT
#define STDCHECK1FLAGS	CHECKED|STDCHECK0FLAGS
#define STDTOGGLE0FLAGS	STDCHECK0FLAGS|MENUTOGGLE
#define STDTOGGLE1FLAGS	STDCHECK1FLAGS|MENUTOGGLE

#define NOSTDMENUFLAGS	STDMENUFLAGS|NM_MENUDISABLED
#define NOSTDITEMFLAGS	STDITEMFLAGS|NM_ITEMDISABLED
#define NOSTDCHECK0FLAGS	STDITEMFLAGS|CHECKIT|MENUTOGGLE|NM_ITEMDISABLED
#define NOSTDCHECK1FLAGS	CHECKED|NOSTDCHECK0FLAGS
#define NOSTDTOGGLE0FLAGS	NOSTDCHECK0FLAGS|MENUTOGGLE
#define NOSTDTOGGLE1FLAGS	NOSTDCHECK1FLAGS|MENUTOGGLE
/*
 * NewMenu: type,*label,*key,flags,exclude,*data
 */

static struct NewMenu NewEditMenu[] =
{
    {NM_TITLE, "PROJECT", NULL, STDMENUFLAGS},
     {NM_ITEM, "NEW", NULL, STDITEMFLAGS},
     {NM_ITEM, "LOAD...", "L", STDITEMFLAGS},
     {NM_ITEM, "SAVE", NULL, STDITEMFLAGS},
     {NM_ITEM, "SAVEAS...", "S", STDITEMFLAGS},
     {NM_ITEM, "QUIT...", "Q", STDITEMFLAGS},
    {NM_TITLE, "TOOL", NULL, STDMENUFLAGS},
     {NM_ITEM, "ACTION", NULL, STDMENUFLAGS},
     {NM_ITEM, NM_BARLABEL,NULL,STDMENUFLAGS},
     {NM_ITEM, "NONE", NULL, STDCHECK1FLAGS, ~(1 << MENU_TOOL_NONE)},
     {NM_ITEM, "VERTEX", NULL, STDCHECK0FLAGS, ~(1 << MENU_TOOL_VERTEX)},
     {NM_ITEM, "SURFACE", NULL, STDCHECK0FLAGS, ~(1 << MENU_TOOL_SURFACE)},
     {NM_ITEM, "LINE", NULL, STDCHECK0FLAGS, ~(1 << MENU_TOOL_LINE)},
     {NM_ITEM, "BALLS", NULL, STDCHECK0FLAGS, ~(1 << MENU_TOOL_BALLS)},
     {NM_ITEM, "GRAB", NULL, STDCHECK0FLAGS, ~(1 << MENU_TOOL_GRAB)},
     {NM_ITEM, "SQUARE", NULL, STDCHECK0FLAGS, ~(1 << MENU_TOOL_SQUARE)},
     {NM_ITEM, "CUBE", NULL, STDCHECK0FLAGS, ~(1 << MENU_TOOL_CUBE)},
     {NM_ITEM, NM_BARLABEL,NULL,STDMENUFLAGS},
     {NM_ITEM, "CANCEL", NULL, STDMENUFLAGS},
    {NM_TITLE, "GRID", NULL, STDMENUFLAGS},
     {NM_ITEM, "1", NULL, STDCHECK1FLAGS, ~(1 << 0)},
     {NM_ITEM, "2", NULL, STDCHECK0FLAGS, ~(1 << 1)},
     {NM_ITEM, "3", NULL, STDCHECK0FLAGS, ~(1 << 2)},
     {NM_ITEM, "4", NULL, STDCHECK0FLAGS, ~(1 << 3)},
     {NM_ITEM, "5", NULL, STDCHECK0FLAGS, ~(1 << 4)},
     {NM_ITEM, "6", NULL, STDCHECK0FLAGS, ~(1 << 5)},
     {NM_ITEM, "7", NULL, STDCHECK0FLAGS, ~(1 << 6)},
     {NM_ITEM, "8", NULL, STDCHECK0FLAGS, ~(1 << 7)},
     {NM_ITEM, "9", NULL, STDCHECK0FLAGS, ~(1 << 8)},
     {NM_ITEM, "10", NULL, STDCHECK0FLAGS, ~(1 << 9)},
     {NM_ITEM, "11", NULL, STDCHECK0FLAGS, ~(1 << 10)},
     {NM_ITEM, "12", NULL, STDCHECK0FLAGS, ~(1 << 11)},
     {NM_ITEM, "13", NULL, STDCHECK0FLAGS, ~(1 << 12)},
     {NM_ITEM, "14", NULL, STDCHECK0FLAGS, ~(1 << 13)},
     {NM_ITEM, "15", NULL, STDCHECK0FLAGS, ~(1 << 14)},
     {NM_ITEM, "16", NULL, STDCHECK0FLAGS, ~(1 << 15)},
     {NM_ITEM, "17", NULL, STDCHECK0FLAGS, ~(1 << 16)},
     {NM_ITEM, "18", NULL, STDCHECK0FLAGS, ~(1 << 17)},
     {NM_ITEM, "19", NULL, STDCHECK0FLAGS, ~(1 << 18)},
     {NM_ITEM, "20", NULL, STDCHECK0FLAGS, ~(1 << 19)},
     {NM_ITEM, "21", NULL, STDCHECK0FLAGS, ~(1 << 20)},
     {NM_ITEM, "22", NULL, STDCHECK0FLAGS, ~(1 << 21)},
     {NM_ITEM, "23", NULL, STDCHECK0FLAGS, ~(1 << 22)},
     {NM_ITEM, "24", NULL, STDCHECK0FLAGS, ~(1 << 23)},
     {NM_ITEM, "25", NULL, STDCHECK0FLAGS, ~(1 << 24)},
     {NM_ITEM, "26", NULL, STDCHECK0FLAGS, ~(1 << 25)},
     {NM_ITEM, "27", NULL, STDCHECK0FLAGS, ~(1 << 26)},
     {NM_ITEM, "28", NULL, STDCHECK0FLAGS, ~(1 << 27)},
     {NM_ITEM, "29", NULL, STDCHECK0FLAGS, ~(1 << 28)},
     {NM_ITEM, "30", NULL, STDCHECK0FLAGS, ~(1 << 29)},
     {NM_ITEM, "31", NULL, STDCHECK0FLAGS, ~(1 << 30)},
     {NM_ITEM, "32", NULL, STDCHECK0FLAGS, ~(1 << 31)},
    {NM_TITLE, "BALL", NULL, STDMENUFLAGS},
     {NM_ITEM, "1", NULL, STDCHECK1FLAGS, ~(1 << 0)},
     {NM_ITEM, "2", NULL, STDCHECK0FLAGS, ~(1 << 1)},
     {NM_ITEM, "3", NULL, STDCHECK0FLAGS, ~(1 << 2)},
     {NM_ITEM, "4", NULL, STDCHECK0FLAGS, ~(1 << 3)},
     {NM_ITEM, "5", NULL, STDCHECK0FLAGS, ~(1 << 4)},
     {NM_ITEM, "6", NULL, STDCHECK0FLAGS, ~(1 << 5)},
     {NM_ITEM, "7", NULL, STDCHECK0FLAGS, ~(1 << 6)},
     {NM_ITEM, "8", NULL, STDCHECK0FLAGS, ~(1 << 7)},
     {NM_ITEM, "9", NULL, STDCHECK0FLAGS, ~(1 << 8)},
     {NM_ITEM, "10", NULL, STDCHECK0FLAGS, ~(1 << 9)},
     {NM_ITEM, "11", NULL, STDCHECK0FLAGS, ~(1 << 10)},
     {NM_ITEM, "12", NULL, STDCHECK0FLAGS, ~(1 << 11)},
     {NM_ITEM, "13", NULL, STDCHECK0FLAGS, ~(1 << 12)},
     {NM_ITEM, "14", NULL, STDCHECK0FLAGS, ~(1 << 13)},
     {NM_ITEM, "15", NULL, STDCHECK0FLAGS, ~(1 << 14)},
     {NM_ITEM, "16", NULL, STDCHECK0FLAGS, ~(1 << 15)},
     {NM_ITEM, "17", NULL, STDCHECK0FLAGS, ~(1 << 16)},
     {NM_ITEM, "18", NULL, STDCHECK0FLAGS, ~(1 << 17)},
     {NM_ITEM, "19", NULL, STDCHECK0FLAGS, ~(1 << 18)},
     {NM_ITEM, "20", NULL, STDCHECK0FLAGS, ~(1 << 19)},
     {NM_ITEM, "21", NULL, STDCHECK0FLAGS, ~(1 << 20)},
     {NM_ITEM, "22", NULL, STDCHECK0FLAGS, ~(1 << 21)},
     {NM_ITEM, "23", NULL, STDCHECK0FLAGS, ~(1 << 22)},
     {NM_ITEM, "24", NULL, STDCHECK0FLAGS, ~(1 << 23)},
     {NM_ITEM, "25", NULL, STDCHECK0FLAGS, ~(1 << 24)},
     {NM_ITEM, "26", NULL, STDCHECK0FLAGS, ~(1 << 25)},
     {NM_ITEM, "27", NULL, STDCHECK0FLAGS, ~(1 << 26)},
     {NM_ITEM, "28", NULL, STDCHECK0FLAGS, ~(1 << 27)},
     {NM_ITEM, "29", NULL, STDCHECK0FLAGS, ~(1 << 28)},
     {NM_ITEM, "30", NULL, STDCHECK0FLAGS, ~(1 << 29)},
     {NM_ITEM, "31", NULL, STDCHECK0FLAGS, ~(1 << 30)},
     {NM_ITEM, "32", NULL, STDCHECK0FLAGS, ~(1 << 31)},
    {NM_TITLE, "EDIT", NULL, STDMENUFLAGS},
     {NM_ITEM, "SELECT   ", NULL, NOSTDITEMFLAGS},
     {NM_ITEM, "DESELECT ", NULL, NOSTDITEMFLAGS},
     {NM_ITEM, "ERASE    ", NULL, NOSTDITEMFLAGS},
     {NM_ITEM, "GRAB     ", NULL, NOSTDITEMFLAGS},
     {NM_ITEM, "COPY     ", NULL, NOSTDITEMFLAGS},
     {NM_ITEM, "SNAP     ", NULL, NOSTDITEMFLAGS},
     {NM_ITEM, "CENTER   ", NULL, NOSTDITEMFLAGS},
    {NM_TITLE, "PREFS", NULL, STDMENUFLAGS},
     {NM_ITEM, "OK-PROMPTS", NULL, NOSTDTOGGLE1FLAGS},
     {NM_ITEM, "X-WINDOW (F1)", NULL, STDTOGGLE1FLAGS},
     {NM_ITEM, "Y-WINDOW (F2)", NULL, STDTOGGLE1FLAGS},
     {NM_ITEM, "Z-WINDOW (F3)", NULL, STDTOGGLE1FLAGS},
     {NM_ITEM, "FILL     (T)", NULL, STDTOGGLE1FLAGS,(1<<5)},
     {NM_ITEM, "LINES    (Y)", NULL, STDTOGGLE0FLAGS,(1<<4)},
     {NM_ITEM, "WORKBENCH", NULL, NOSTDTOGGLE1FLAGS},
     {NM_ITEM, "PENS...", NULL, NOSTDITEMFLAGS},
     {NM_ITEM, "PALETTE...", NULL, NOSTDITEMFLAGS},
    {NM_END}
};

static struct TagItem editmenutags[] =
{
    {TAG_END}
};

static struct menudata editmenudata =
{
    NULL,
    NULL,
    NewEditMenu,
    editmenutags
};

void seteditmenu(struct windowdata * windat)
{
    setmenu(windat, &editmenudata);
}

void freeeditmenu(void)
{
    freemenu(&editmenudata);
}

void createeditmenu(void)
{
    createmenu(&editmenudata);
}

void seteditmenustate(int menu, int item, int subitem, int state)
{
    setmenustate(&editmenudata, menu, item, subitem, state);
}

int geteditmenustate(int menu, int item, int subitem)
{
    return getmenustate(&editmenudata, menu, item, subitem);
}
