/*
 * source/edit/edit_ready.c
 * SpacePigs VectorMania Edit part
 *
 * 1991-07-19 : Coding begun
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "other/debug.h"
#include "vector/vector.h"
#include "graphics/graphics.h"
#include "edit/edit.h"
#include "edit/edit_globals.h"

/******* namngivningen av funktionerna ***********
 * drawedit<xxxx>  skalar om, och ritar ut vad de skall (anropar editdraw<xxx>)
 * editdraw<xxxx>  tar omskalade kordinater och ritar ut i de olika fnstren
 * draw<xxxx>      ritar ut i valt fnster efter given axel
 */

/****** Close Edit *********
 * void colseedit(void)
 * terstll efter editorn
 */
void closeedit(void)
{
    closewindow(&editxwindow);
    closewindow(&editywindow);
    closewindow(&editzwindow);
    freeeditmenu();
}

/*** Update Edit Display ****
 * Uppdatera skrmen
 */
void updateeditdisplay(void)
{
    if (selectwindow(&editxwindow))
	clearscreen();
    if (selectwindow(&editywindow))
	clearscreen();
    if (selectwindow(&editzwindow))
	clearscreen();
    draweditobject();
    draweditcursor();
    drawedittooldraw();
}

/****** Draw edit object ***
 * void draweditobject(void)
 * rita objectet i editfnstren
 */
void draweditobject(void)
{
    editdisplayscalecords();
    editdrawdraw(editdispcord, editdraw);
    editdrawpoints(editdispcord, editcordstat, numeditcords);
}

/****** Draw edit draw *****
 * void draweditdraw(void)
 * skala om, och rita alla element i objectet
 */
void draweditdraw(void)
{
    editdisplayscalecords();
    editdrawdraw(editdispcord, editdraw);
}

/****** Draw edit points *****
 * void draweditpoints (void)
 * skala om, och rita alla punkter i objectet, markerar valda punkter
 */
void draweditpoints(void)
{
    editdisplayscalecords();
    editdrawpoints(editdispcord, editcordstat, numeditcords);
}

/****** Draw edit point ********
 * void draweditpoint (int cord);
 * skalar om, och rita ut en punkt, markerar om den r vald.
 */
void draweditpoint(int cord)
{
    if (cord < 0)
	return;
    editdisplayscalecords();
    editdrawpoint(&editdispcord[cord], editcordstat[cord]);
}

/****** Draw edit selected point *****
 * void draweditselpoint (int cord)
 * skalar om, och ritar ut en punkt som vald.
 */
void draweditselpoint(int cord)
{
    if (cord < 0)
	return;
    editdisplayscalecords();
    editdrawselpoint(&editdispcord[cord]);
}

/****** Draw edit selected points *****
 * void draweditselpoints (void)
 * skalar om, och ritar ut alla valda punkter
 */
void draweditselpoints(void)
{
    editdisplayscalecords();
    editdrawselpointlist(editdispcord, editcordselect, numselect);
}

void draweditobj3d(void)
{
    if (!selectwindow(DRAWWINDOW)) {
	saydebug("draweditobj: Can't get drawwindow???\n", 0);
	return;
    }
    copycords(editcord, editcord3d, numeditcords);
    rotcords(editcord3d, editcord3d, numeditcords, X_AXEL, editobj_rx);
    rotcords(editcord3d, editcord3d, numeditcords, Y_AXEL, editobj_ry);
    rotcords(editcord3d, editcord3d, numeditcords, Z_AXEL, editobj_rz);
    perspectivecords(editcord3d, editcord3d, numeditcords);
    setcordbas(editcord3d);
    makeelems(editdraw);
    clearscreen();
    draw_elements();
    free_elements();
}

void editdrawdraw(CORD3D cords[], char drawlist[])
{
    if (selectwindow(&editxwindow))
	drawdraw(cords, drawlist, X_AXEL);
    if (selectwindow(&editywindow))
	drawdraw(cords, drawlist, Y_AXEL);
    if (selectwindow(&editzwindow))
	drawdraw(cords, drawlist, Z_AXEL);
}

void editdrawpoint(CORD3D * cord, char flags)
{
    if (selectwindow(&editxwindow)) {
	cursoroff();
	drawpoint(cord, flags, X_AXEL);
	cursoron();
    }
    if (selectwindow(&editywindow)) {
	cursoroff();
	drawpoint(cord, flags, Y_AXEL);
	cursoron();
    }
    if (selectwindow(&editzwindow)) {
	cursoroff();
	drawpoint(cord, flags, Z_AXEL);
	cursoron();
    }
}

void editdrawpoints(CORD3D cords[], char flags[], int numcords)
{
    setcolor(colorpoint);
    if (selectwindow(&editxwindow)) {
	cursoroff();
	drawpoints(cords, flags, numcords, X_AXEL);
	cursoron();
    }
    if (selectwindow(&editywindow)) {
	cursoroff();
	drawpoints(cords, flags, numcords, Y_AXEL);
	cursoron();
    }
    if (selectwindow(&editzwindow)) {
	cursoroff();
	drawpoints(cords, flags, numcords, Z_AXEL);
	cursoron();
    }
}

void editdrawselpoint(CORD3D * cord)
{
    if (selectwindow(&editxwindow)) {
	cursoroff();
	drawselpoint(cord, X_AXEL);
	cursoron();
    }
    if (selectwindow(&editywindow)) {
	cursoroff();
	drawselpoint(cord, Y_AXEL);
	cursoron();
    }
    if (selectwindow(&editzwindow)) {
	cursoroff();
	drawselpoint(cord, Z_AXEL);
	cursoron();
    }
}

void editdrawselpointlist(CORD3D cords[], char *pointlist, char listlen)
{
    if (selectwindow(&editxwindow)) {
	cursoroff();
	drawselpointlist(cords, pointlist, listlen, X_AXEL);
	cursoron();
    }
    if (selectwindow(&editywindow)) {
	cursoroff();
	drawselpointlist(cords, pointlist, listlen, Y_AXEL);
	cursoron();
    }
    if (selectwindow(&editzwindow)) {
	cursoroff();
	drawselpointlist(cords, pointlist, listlen, Z_AXEL);
	cursoron();
    }
}

void drawpoint(CORD3D * cord, char flags, enum Axel axel)
{
    CORD3D point;
    geteditcord(cord, &point, axel);
    if (flags & CORD_SELECTED)
	dispselpoint(&point);
    else
	disppoint(&point);
}

void drawpoints(CORD3D cords[], char flags[], int numcords, enum Axel axel)
{
    int cord;

    for (cord = 0; cord < numcords; cord++) {
	drawpoint(&cords[cord], flags[cord], axel);
    }
}

void drawselpoint(CORD3D * cord, enum Axel axel)
{
    CORD3D point;

    geteditcord(cord, &point, axel);
    dispselpoint(&point);
}

void drawselpointlist(CORD3D cords[], char pointlist[], char listlen, enum Axel axel)
{
    int cord;

    for (cord = 0; cord < listlen; cord++) {
	drawselpoint(&cords[cord], axel);
    }
}

void disppoint(CORD3D * point)
{
    setcolor(colorpoint);
    move(point);
    pset(point);
}

void dispselpoint(CORD3D * point)
{
    setcolor(colorselpoint);
    move(point);
    psetdelta(0, 0);
    psetdelta(-1, 0);
    psetdelta(0, 1);
    psetdelta(1, 0);
    psetdelta(0, -1);
}

void draweditcursor(void)
{
    CORD3D scursor;

    displayscalecords(&cursor, &scursor, 1);

    setcolor(colorcursor);
    if (selectwindow(&editxwindow))
	drawcursor(scursor.z, scursor.y);
    if (selectwindow(&editywindow))
	drawcursor(scursor.x, scursor.z);
    if (selectwindow(&editzwindow))
	drawcursor(scursor.x, scursor.y);
}

void drawcursor(int x, int y)
{
    CORD3D cord;
    cord.x = x;
    cord.y = y;
    cord.z = 0;

    setcursor(&cord);
}

int createvertex(CORD3D * cord)
{
    VECTINT x, y, z;
    int num;

    x = cord->x;
    y = cord->y;
    z = cord->z;
    num = numeditcords++;
    if (numeditcords > EDITCORD_SIZE) {
	saydebug("Create vertex, vertexbuffer full(%d).\n", num);
	numeditcords = EDITCORD_SIZE;
	return -1;
    }

    editcord[num].x = x;
    editcord[num].y = y;
    editcord[num].z = z;
    editcordstat[num] = 0;
    draweditpoint(num);
    return num;
}

void editdisplayscalecords(void)
{
    displayscalecords(editcord, editdispcord, numeditcords);
}

void displayscalecords(CORD3D * source, CORD3D * dest, int numcords)
{
    transcords(source, dest, numcords, &center);
    scalecords(dest, dest, numcords, scale);
}

int findnear(CORD3D * cords, int numcords, CORD3D * cord, enum Axel axel, int limit, VECTFLOAT zfactor)
{
    int i;
    VECTINT x, y, z;
    int limit2 = limit * limit;
    int minlimit = limit2;
    int mincord = -1;
    CORD3D point;

    geteditcord(cord, &point, axel);
    x = point.x;
    y = point.y;
    z = point.z;

    for (i = 0; i < numcords; i++) {
	VECTINT xa, ya, za;
	int hyp;
	if (editcordstat[i] & CORD_NOSNAP)
	    continue;
	geteditcord(&cords[i], &point, axel);
	xa = ABS(point.x - x);
	if (xa > limit)
	    continue;
	ya = ABS(point.y - y);
	if (ya > limit)
	    continue;
	za = IMULIF(ABS(point.z - z), zfactor);
	if (za > limit)
	    continue;
	hyp = xa * xa + ya * ya + za * za;
	if (hyp > limit2)
	    continue;
	if (hyp > minlimit)
	    continue;
	mincord = i;
	minlimit = hyp;
    }
    return mincord;
}

int findnearcursor(WINDOWTYPE type)
{
    enum Axel axel;
    CORD3D cur;
    int cord;

    switch (type) {
    case WINDOW_X:
	axel = X_AXEL;
	break;
    case WINDOW_Y:
	axel = Y_AXEL;
	break;
    case WINDOW_Z:
	axel = Z_AXEL;
	break;
    default:
	saydebug("findnearcursor: unknown windowtype %d\n", type);
	return -1;
    }
    displayscalecords(&cursor, &cur, 1);
    cord = findnear(editdispcord, numeditcords, &cur, axel, FINDLIMIT, FINDZFACTOR);
    if (cord < 0) {
	return -1;
    }
    return cord;
}

void setcursorpos(CORD3D * cord)
{
    cursor.x = cord->x;
    cursor.y = cord->y;
    cursor.z = cord->z;
    draweditcursor();
}

void snapcursorcord(int cord)
{
    setcursorpos(&editcord[cord]);
}

int snapcursornearest(WINDOWTYPE type)
{
    int cord;

    cord = findnearcursor(type);
    if (cord >= 0)
	snapcursorcord(cord);
    return cord;
}

void geteditcord(CORD3D * source, CORD3D * dest, enum Axel axel)
{
    VECTINT x, y, z;

    x = source->x;
    y = source->y;
    z = source->z;

    switch (axel) {
    case X_AXEL:
	dest->x = z;
	dest->y = y;
	dest->z = x;
	break;
    case Y_AXEL:
	dest->x = x;
	dest->y = z;
	dest->z = y;
	break;
    case Z_AXEL:
	dest->x = x;
	dest->y = y;
	dest->z = z;
	break;
    default:
	saydebug("geteditcord: unknown axel %d\n", axel);
    }
}

void
 drawdraw_surf(CORD3D cords[], STDDRAWTYPE *elemdraw, enum Axel axel)
{
    CORD3D cord;
    CORD3D start;
    int numcords, i;
    char *cordlist;

    setcolor(colorsurf);
    numcords = elemdraw->numcords;
    cordlist = elemdraw->cords;
    geteditcord(&cords[*cordlist++], &start, axel);
    move(&start);
    for (i = 1; i < numcords; i++) {
	geteditcord(&cords[*cordlist++], &cord, axel);
	draw(&cord);
    }
    draw(&start);
}

void
 drawdraw_line(CORD3D cords[], STDDRAWTYPE *elemdraw, enum Axel axel)
{
    CORD3D cord;
    CORD3D start;
    unsigned char *cordlist=elemdraw->cords;
    
    setcolor(colorline);
    geteditcord(&cords[*cordlist++], &start, axel);
    move(&start);
    geteditcord(&cords[*cordlist++], &cord, axel);
    draw(&cord);
}

void
 drawdraw_ball(CORD3D cords[], STDDRAWTYPE *elemdraw, enum Axel axel)
{
    CORD3D cord;
    int radie;
    unsigned char *cordlist=elemdraw->cords;
    BALLDRAWDATA *data=(BALLDRAWDATA *)GETDRAWDATA(elemdraw);
    
    setcolor(colorball);
    radie = data->radie;
    radie = IMULIF(radie, scale);
    geteditcord(&cords[*cordlist], &cord, axel);
    circle(&cord, radie);
}

STDDRAWTYPE *
 drawdrawelem(CORD3D cords[], STDDRAWTYPE *elemdraw, enum Axel axel)
{
    switch (elemdraw->type) {
    case DT_LINE:
	drawdraw_line(cords, elemdraw, axel);
	break;
    case DT_SURF:
    case DT_SURFFRONT:
	drawdraw_surf(cords, elemdraw, axel);
	break;
    case DT_BALL:
	drawdraw_ball(cords, elemdraw, axel);
	break;
    case DT_END:
	return (char *) 0L;
    }
    return NEXTDRAW(elemdraw);
}

void drawdraw(CORD3D cords[], char drawlist[], enum Axel axel)
{
    STDDRAWTYPE *elemdraw=(STDDRAWTYPE *)drawlist;

    while (elemdraw) {
	elemdraw = drawdrawelem(cords, elemdraw, axel);
    }
}

void movecenter(GADGETID id, WINDOWTYPE type)
{
    VECTINT step;
    VECTINT x, y;
    VECTINT mx, my, mz;
    CORD3D newcent;

    step = IDIVIF(MOVECENTERSTEP, scale);
    x = 0;
    y = 0;
    switch (id) {
    case GADGET_UP:
	y = step;
	break;
    case GADGET_DOWN:
	y = -step;
	break;
    case GADGET_LEFT:
	x = -step;
	break;
    case GADGET_RIGHT:
	x = step;
	break;
    }
    mx = 0;
    my = 0;
    mz = 0;
    switch (type) {
    case WINDOW_X:
	mz = x;
	my = y;
	break;
    case WINDOW_Y:
	mx = x;
	mz = y;
	break;
    case WINDOW_Z:
	mx = x;
	my = y;
	break;
    }
    cursor.x+=mx;
    cursor.y+=my;
    cursor.z+=mz;
    
    newcent.x = mx-center.x;
    newcent.y = my-center.y;
    newcent.z = mz-center.z;

    setcenter(&newcent);
}

void setcenter(CORD3D * newcent)
{
    center.x = -newcent->x;
    center.y = -newcent->y;
    center.z = -newcent->z;
}

void zoomin(void)
{
    scale = MULFF(scale, ZOOMFACTOR);
    if (scale > ZOOMMAX)
	scale = ZOOMMAX;
}

void zoomout(void)
{
    scale = DIVFF(scale, ZOOMFACTOR);
    if (scale < ZOOMMIN)
	scale = ZOOMMIN;
}

void selectnearcursor(WINDOWTYPE type)
{
    int cord;

    cord = findnearcursor(type);
    if (cord >= 0)
	selectcord(cord);
}

void selectcord(int cord)
{
    if (cord < 0)
	return;
    if (editcordstat[cord] & CORD_SELECTED) {
	saydebug("Cord:%d already selected\n", cord);
	return;
    }
    editcordselect[numselect++] = cord;
    editcordstat[cord] |= CORD_SELECTED;
    draweditselpoint(cord);
}

void deselectcord(int cord)
{
    int i;
    if (cord < 0)
	return;

    if (!editcordstat[cord] & CORD_SELECTED) {
	saydebug("Tries to deselect unselected cord:%d\n", cord);
	return;
    }
    editcordstat[cord] &= ~CORD_SELECTED;
    i = 0;
    while (editcordselect[i] != cord) {
	i += 1;
	if (i >= numselect)
	    break;
    }
    if (i >= numselect) {
	saydebug("deselect can't find cord:%d\n", cord);
	return;
    }
    for (; i < numselect; i++) {
	editcordselect[i] = editcordselect[i + 1];
    }
    numselect -= 1;
    if (numselect < 0) {
	saydebug("deselectcord: %d ", cord);
	saydebug("ERROR: numselect < 0 (=%d)\n", numselect);
	numselect = 0;
    }
}

STDDRAWTYPE *
createdraw(DRAWTYPE type,int numcords,char cords[],int color,int datasize,STDDRAWDATA *data)
{
    STDDRAWTYPE *elemdraw=(STDDRAWTYPE *)&editdraw[nextdraw];
    int size=CALCDRAWSIZE(numcords,datasize);
    int i;
    char *drawcords;
    
    if((nextdraw+size)>= EDITDRAW_SIZE) {
	saydebug("Create draw: drawtable full:%d\n", type);
	return 0;
    }
    elemdraw->type=type;
    elemdraw->size=size;
    elemdraw->numcords=numcords;
    elemdraw->color=color;
    drawcords=elemdraw->cords;
    for(i=0;i<numcords;i++) {
    	*drawcords++=*cords++;
    }
    if(datasize) {
    	char *drawdataptr,*dataptr;
    	drawdataptr=(char *)GETDRAWDATA(elemdraw);
    	dataptr=(char *)data;
    	for(i=0;i<datasize;i++) {
    		*drawdataptr++=*dataptr++;
    	}
    }
    
    nextdraw+=size;
    editdraw[nextdraw]=DT_END;
    
    return elemdraw;
}    
    	
void createsurf(char cordlist[], int listlen)
{
    if (listlen < 3) {
	saydebug("Create surf requires at least three cords!\n", 0);
	return;
    }
    createdraw(DT_SURF,listlen,cordlist,colorsurf,0,0);
}

void createline(int start, int end)
{
    char cordlist[2];
    cordlist[0]=start;
    cordlist[1]=end;
    
    createdraw(DT_LINE,2,cordlist,colorline,0,0);
}

void createlinelist(char linelist[], int listlen)
{
    int i, end;
    if (listlen < 2) {
	saydebug("createlinelist requires at least two cords!\n", 0);
	return;
    }
    end = listlen - 1;
    for (i = 0; i < end; i++) {
	createline(linelist[i], linelist[i + 1]);
    }
}

/****** Create Ball ************
 * void createball (int pos, int radie)
 * skapar en boll med given radie
 */
void createball(int pos, int radie)
{
    char cords[1];
    BALLDRAWDATA data;
    
    cords[0]=pos;
    data.radie=radie;
    createdraw(DT_BALL,1,cords,colorball,sizeof(BALLDRAWDATA),(STDDRAWDATA *)&data);
}

/****** Create ball list ******
 * void createballlist (char balllist[], int listlen)
 * skapar en serie med bollar, med samma radie
 */
void createballlist(char balllist[], int listlen)
{
    int ball;

    if (listlen < 1) {
	saydebug("Create balls: Need 1 or more points:%d\n", listlen);
	return;
    }
    for (ball = 0; ball < listlen; ball++) {
	createball(balllist[ball], editballradie);
    }
}

void deselectall(void)
{
    int i;
    int cord;

    for (i = 0; i < numselect; i++) {
	cord = editcordselect[i];
	editcordstat[cord] &= ~CORD_SELECTED;
    }
    numselect = 0;
}

void eraseelem(int elem)
{
    int i;
    int size;
    STDDRAWTYPE *elemdraw=(STDDRAWTYPE *)editdraw[elem];
    
    if (elem >= nextdraw) {
	saydebug("Cant erase element outside drawlist!!:%d\n", elem);
	return;
    }
    size=elemdraw->size;
    
    nextdraw -= size;
    for (i = elem; i <= nextdraw; i++) {
	editdraw[i] = editdraw[i + size];
    }
}

int findelem(char cordlist[], int listlen, int start)
{
    int i;
    int code;
    int found;
    
    STDDRAWTYPE *elemdraw=(STDDRAWTYPE *)editdraw;
    
    if (start >= listlen) {
	saydebug("findelem, start at end of list:%d\n", start);
	return -1;
    }
    if (listlen == 0) {
	saydebug("findelem, nocords\n", 0);
	return -1;
    }
    found = 0;
    i = 0;
    while(elemdraw->type!=DT_END) {
	code = checkifelem(elemdraw->cords, elemdraw->numcords, cordlist, listlen);
	if (code > 0)
	    return (int)((unsigned int)elemdraw-(unsigned int)editdraw);
	if (code < 0)
	    found = 1;
	elemdraw=NEXTDRAW(elemdraw);
    }
    if (found)
	return -1;		/* partial match */
    return -2;			/* no element match */
}

int checkifelem(char elemlist[], int numcords, char cordlist[], int listlen)
{
    int found;
    int i, e;
    char cord;

    if (listlen == 0 || numcords == 0)
	return 0;

    found = 0;
    for (i = 0; i < listlen; i++) {
	cord = cordlist[i];
	for (e = 0; e < numcords; e++) {
	    if (cord == elemlist[e]) {
		found += 1;
		break;
	    }
	}
    }

    if (found == listlen)
	return 1;		/* element match all selected! */
    if (found == numcords)
	return 2;		/* all elementpoints selected! */
    if (found > 0)
	return -1;		/* partial match.. */
    return 0;			/* no match */
}

int setgrab(int state, WINDOWTYPE type)
{
    int newstate;
    
    newstate=setgrabstat(state, type);
    
    graborgin.x = cursor.x;
    graborgin.y = cursor.y;
    graborgin.z = cursor.z;
    
    return newstate;
}

int setgrabstat(int state, WINDOWTYPE type)
{
    int cord;
    int cstat;

    /* Grab ON */
    if (state) {
	if (numselect == 0) {
	    /* Grab nrmsta enbart. */
	    cord = findnearcursor(type);
	    if (cord >= 0) {
	    	snapcursorcord(cord);
		editcordstat[cord] |= CORD_GRABFLAGS;
		grabcord[cord] = editcord[cord];
		return 1;
	    }
	} else {
	    /* grab alla valda */
	    for (cord = 0; cord < numeditcords; cord++) {
		cstat = editcordstat[cord];
		if (cstat & CORD_SELECTED)
		    cstat |= CORD_GRABFLAGS;
		editcordstat[cord] = cstat;
		grabcord[cord] = editcord[cord];
	    }
	    return 1;
	}
    } else {
	/* Grab OFF */
	for (cord = 0; cord < numeditcords; cord++) {
	    editcordstat[cord] &= ~CORD_GRABFLAGS;
	}
    }
    return 0;
}

void grab_cancel(void)
{
    int cord;
    for (cord = 0; cord < numeditcords; cord++) {
	if (editcordstat[cord] & CORD_GRABFLAGS) {
	    editcord[cord] = grabcord[cord];
	    editcordstat[cord] &= ~CORD_GRABFLAGS;
	}
    }
}

void movegrab(CORD3D * orgin, CORD3D * dest)
{
    VECTINT dx, dy, dz;
    int cord;

    dx = dest->x - orgin->x;
    dy = dest->y - orgin->y;
    dz = dest->z - orgin->z;
    orgin->x = dest->x;
    orgin->y = dest->y;
    orgin->z = dest->z;

    for (cord = 0; cord < numeditcords; cord++) {
	if (editcordstat[cord] & CORD_GRAB) {
	    editcord[cord].x += dx;
	    editcord[cord].y += dy;
	    editcord[cord].z += dz;
	}
    }
}

void editmovegrab(CORD3D * pos)
{
    movegrab(&graborgin, pos);
}

/********* flip elem ***********
 * void flipelem (int elem)
 * ndrar bak/framsida p en yta
 */
void flipelem(int elem)
{
    int num, numcords;
    int i;
    char *start;
    char *end;
    
    STDDRAWTYPE *elemdraw=(STDDRAWTYPE *)&editdraw[elem];
    
    switch (elemdraw->type) {
    case DT_LINE:
    case DT_BALL:
    	return;
    case DT_SURF:
    case DT_SURFFRONT:
	numcords = elemdraw->numcords;
	start = elemdraw->cords;
	break;
    default:
	saydebug("Flip elem, unknown type:%d\n", editdraw[elem]);
	return;
    }
    end = start+numcords-1;
    num = numcords / 2;
    for (i = 0; i < num; i++) {
	char temp;
	temp = *start;
	*start++ = *end;
	*end-- = temp;
    }
}

/******** Edit flip elem *********
 * void editflipelem (void)
 * ndrar bak/framsida p valda ytor
 * alla valda punkter mste ing i respektive yta.
 */
void editflipelem(void)
{
    int elem;

    for (elem = 0; (elem = findelem(editcordselect, numselect, elem)) >= 0;) {
	if (elem == -1)
	    saydebug("editflip: Found partial element(s)\n", 0);
	if (elem == -2)
	    saydebug("editflip: Found nothing\n", 0);
	if (elem >= 0) {
	    saydebug("editflip: found element %p\n", elem);
	    flipelem(elem);
	}
    }
}

void roteditobj(int step, enum Axel axel)
{
    switch (axel) {
	case X_AXEL:
	editobj_rx += step;
	while (editobj_rx > 360)
	    editobj_rx -= 360;
	while (editobj_rx < 0)
	    editobj_rx += 360;
	break;
    case Y_AXEL:
	editobj_ry += step;
	while (editobj_ry > 360)
	    editobj_ry -= 360;
	while (editobj_ry < 0)
	    editobj_ry += 360;
	break;
    case Z_AXEL:
	editobj_rz += step;
	while (editobj_rz > 360)
	    editobj_rz -= 360;
	while (editobj_rz < 0)
	    editobj_rz += 360;
	break;
    default:
	saydebug("roteditobj: unknown axel:%d\n", axel);
	break;
    }
}

void clearallcordflags(void)
{
	int i;
	char *flags;
	flags=editcordstat;
	for(i=0;i<EDITCORD_SIZE;i++) {
		*flags++=0;
	}
}

/***** Clear edit obj ******
 * void cleareditobj (void)
 * slnger bort objectet ur minnet.
 * utfrs vid tex NEW
 */
void cleareditobj(void)
{
    CORD3D cent =
    {0, 0, 0};
    setgrab(0, 0L);
    deselectall();
    clearallcordflags();
    resettools();
    scale = ZOOMDEFAULT;
    numeditcords = 0;
    realnumcords = 0;
    nextdraw = 0;
    editdraw[nextdraw] = DT_END;
    updateeditdisplay();
    setcenter(&cent);
    setcursorpos(&cent);
    reseteditobj3d();
}

/***** new edit obj *******
 * terstller objecteditorns data till default
 * slnger bord objectet, och stter namnet till namnlst
 */
void neweditobj(void)
{
    cleareditobj();
    strcpy(objfilename, OBJUNNAMED);
}

void saveeditobj(char *namn)
{
    FILE *fil;
    short num;
    
    if (namn == CURRENTFILENAME) {
	namn = objfilename;
    }
    if (namn == REQUESTFILENAME) {
	namn = requestfile("Objects/", objfilename, "~(#?.info)", 0);
    }
    if (namn && strcmp(namn, OBJUNNAMED) == 0) {
	namn = requestfile("Objects/", objfilename, "~(#?.info)", 0);
    }
    if (namn) {
	strncpy(objfilename, namn, OBJFILENAMEMAX);

	fil = fopen(objfilename, "wb");
	if (!fil) {
	    saydebug("Cant open file for save:%s\n", (int) objfilename);
	    return;
	}
	fwrite("SPVMOB02", 8, 1, fil);
	num=numeditcords;
	fwrite((char *)&num,2,1,fil);
	fwrite((char *) editcord, sizeof(CORD3D), numeditcords, fil);
	num=nextdraw;
	fwrite((char *)&num,2,1,fil);
	fwrite(editdraw, sizeof(char), nextdraw, fil);
	fclose(fil);
    }
}

void loadeditobj(char *namn)
{
    FILE *fil;
    char typebuf[8];
    int ver;
    short num;
    
    if (namn == CURRENTFILENAME) {
	namn = objfilename;
    }
    if (namn == REQUESTFILENAME) {
	namn = requestfile("Objects/", objfilename, "~(#?.info)", 0);
    }
    if (namn) {
    	cleareditobj();
	strncpy(objfilename, namn, OBJFILENAMEMAX);

	fil = fopen(objfilename, "rb");
	if (!fil) {
	    saydebug("Cant open file for load:%s\n", (int) objfilename);
	    return;
	}
	fread(typebuf, 8, 1, fil);
	if (memcmp("SPVMOB", typebuf, 6) != 0) {
	    saydebug("Not an SPVMOB file??\n", 0);
	    return;
	}
	ver=(typebuf[6]-'0')*10+typebuf[7]-'0';
	cleareditobj();
	
	if(ver==0||ver==1) numeditcords = fgetc(fil);
	else {
		fread((char *)&num,2,1,fil);
		numeditcords=num;
	}
	fread((char *) editcord, sizeof(CORD3D), numeditcords, fil);
	if(ver==0) {
		char *tempdraw,*tempptr;
		int tempsize=fgetc(fil);
		tempptr=tempdraw=malloc(tempsize+1);
		fread(tempdraw,sizeof(char),tempsize,fil);
		tempdraw[tempsize]=DT_END;
		while(*tempdraw!=DT_END) {
			switch(*tempdraw++) {
			case DT_LINE:
				createline(*tempdraw++,*tempdraw++);
				break;
			case DT_SURF:
			case DT_SURFFRONT:
				createsurf(tempdraw+1,*tempdraw);
				tempdraw+=1+*tempdraw;
				break;
			case DT_BALL:
				createball(tempdraw[1],*tempdraw);
				tempdraw+=2;
				break;
			}
		}
		free(tempptr);
	} else {
		if(ver==1) num = fgetc(fil);
		else fread((char *)&num,2,1,fil);
		nextdraw=num;
		fread(editdraw, sizeof(char), nextdraw, fil);
		editdraw[nextdraw] = DT_END;
	}

	fclose(fil);
	updateeditdisplay();
	reseteditobj3d();
    }
}

void rescaleeditcord(int *wx, int *wy, WINDOWDATA * windat)
{
    int x = *wx, y = *wy;
    int nx, ny;
    int negx;
    int negy;

    ny = windat->ycenter - y;
    nx = x - windat->xcenter;
    ny = IDIVIF(ny, XYRATIO)+(ny>0?1:0); /* knas-justering... orkar inte strula med avrundning osv..*/
    negx = nx < 0 ? 1 : 0;
    nx = ABS(nx);
    negy = ny < 0 ? 1 : 0;
    ny = ABS(ny);

    ny = RIDIVIF(ny, scale);
    nx = RIDIVIF(nx, scale);

    if (negx)
	nx = -nx;
    if (negy)
	ny = -ny;
    *wx = nx;
    *wy = ny;
}

static WINDOWTYPE lasttype = -1;
static int lastx, lasty;

int checkifeditdoubleclick(int x, int y, WINDOWTYPE type)
{
    int dclick = 0;
    
    if (lasttype == type) {
	if ((ABS(lastx - x) < DCLICKLIMIT) && (ABS(lasty - y) < DCLICKLIMIT)) {
	    dclick = 1;
	}
    }
    return dclick;
}

int checkeditdoubleclick(int x, int y, WINDOWTYPE type)
{
    int dclick;
    dclick = checkifeditdoubleclick(x, y, type);
    lastx = x;
    lasty = y;
    lasttype = type;
    return dclick;
}

void reseteditobj3d(void)
{
    editobj_rx = 0;
    editobj_ry = 0;
    editobj_rz = 0;
    draweditobj3d();
}


