/*
 * source/edit/edit_idcmp.c
 * SpacePigs VectorMania Edit part
 *
 * 1991-07-19 : Coding begun
 */

#include "other/debug.h"
#include "vector/vector.h"
#include "graphics/graphics.h"
#include "edit/edit.h"
#include "edit/edit_globals.h"

void activewindow(struct windowdata * windat)
{
    windat->flags |= WINFLAG_NEWACTIVE;
    menu_on(windat);
}

void inactivewindow(struct windowdata * windat)
{
    menu_off(windat);
}

int mouseselectdown(int x, int y, int doubleclick, struct windowdata * windat)
{
    static struct windowdata *lastwin = 0;
    static int lastx = ~0 >> 1, lasty = ~0 >> 1;
    static int lastdoubleclick = 0;

    int nx = x, ny = y;
    VECTINT cx, cy, cz;
    int moveflag = 1;


    if (doubleclick && windat->type != WINDOW_DRAW) {
	if (!lastdoubleclick) {
	    if (snapcursornearest(windat->type) >= 0) {
		tool_move(&cursor);
	    }
	}
	moveflag = 0;
    }
    if (lastwin == windat && lastx == x && lasty == y) {
	moveflag = 0;
    }
    if (moveflag) {
	cx = cursor.x + center.x;
	cy = cursor.y + center.y;
	cz = cursor.z + center.z;

	switch (windat->type) {
	case WINDOW_DRAW:
	    return 1;
	case WINDOW_X:
	    cz = nx;
	    cy = ny;
	    break;
	case WINDOW_Y:
	    cx = nx;
	    cz = ny;
	    break;
	case WINDOW_Z:
	    cx = nx;
	    cy = ny;
	    break;
	}
	cursor.x = cx - center.x;
	cursor.y = cy - center.y;
	cursor.z = cz - center.z;
	if (grid > 1) {
	    snapgrid(&cursor);
	}
	tool_move(&cursor);
	draweditcursor();
    }
    lastwin = windat;
    lastx = x;
    lasty = y;
    lastdoubleclick = doubleclick;
    return 1;
}

int mouseselectup(int x, int y, int doubleclick, struct windowdata * windat)
{
    return 1;
}

int mousemenudown(int x, int y, int doubleclick, struct windowdata * windat)
{
    mouseselectdown(x, y, doubleclick, windat);
    tool_action(&cursor, doubleclick, windat);
    return 1;
}

int mousemenuup(int x, int y, int doubleclick, struct windowdata * windat)
{
    return 1;
}

int gadgetdown(GADGETID id, int selected, int doubleclick, struct windowdata * windat)
{
    WINDOWTYPE win = windat->type;
    switch (id) {
    case GADGET_GRAB:
	settooltype(TOOL_GRAB, win);
	break;
    case GADGET_VERTEX:
	settooltype(TOOL_VERTEX, win);
	return 1;
    case GADGET_SURFACE:
	settooltype(TOOL_SURFACE, win);
	break;
    case GADGET_LINE:
	settooltype(TOOL_LINE, win);
	break;
    case GADGET_BALL:
	settooltype(TOOL_BALL, win);
	break;
    }
    return 1;
}

int gadgetup(GADGETID id, int selected, int doubleclick, struct windowdata * windat)
{
    switch (id) {
	case GADGET_UP:
	case GADGET_DOWN:
	case GADGET_LEFT:
	case GADGET_RIGHT:
	movecenter(id, windat->type);
	break;
    case GADGET_ZOOMIN:
	zoomin();
	break;
    case GADGET_ZOOMOUT:
	zoomout();
	break;
    case GADGET_ERASE:
	erase();
	break;
    case GADGET_FLIP:
	editflipelem();
	return 1;
    default:
	saydebug("Unhandled gadget: %d\n", id);
	break;
    }
    updateeditdisplay();
    return 1;
}

extern int fillmode;

int handlemenu_end(struct windowdata * windat)
{
	int winstat;
	int menustat;

	/* x-window */
	winstat=(selectwindow(&editxwindow)!=0);
	menustat=geteditmenustate(MENU_PREFS,MENU_PREFS_X,~0);
	if(winstat&&!menustat) closewindow(&editxwindow);
	if(!winstat&&menustat) {
		openwindow(&editxwindow);
		updateeditdisplay();
	}
	
	/* y-window */
	winstat=(selectwindow(&editywindow)!=0);
	menustat=geteditmenustate(MENU_PREFS,MENU_PREFS_Y,~0);
	if(winstat&&!menustat) closewindow(&editywindow);
	if(!winstat&&menustat) {
		openwindow(&editywindow);
		updateeditdisplay();
	}
	
	/* y-window */
	winstat=(selectwindow(&editzwindow)!=0);
	menustat=geteditmenustate(MENU_PREFS,MENU_PREFS_Z,~0);
	if(winstat&&!menustat) closewindow(&editzwindow);
	if(!winstat&&menustat) {
		openwindow(&editzwindow);
		updateeditdisplay();
	}
	
	/* fillmode */
	menustat=geteditmenustate(MENU_PREFS,MENU_PREFS_FILL,~0);
	if(menustat&&fillmode!=FILL_MODE_FILL) {
		setfillmode(FILL_MODE_FILL);
		draweditobj3d();
	}
	menustat=geteditmenustate(MENU_PREFS,MENU_PREFS_LINES,~0);
	if(menustat&&fillmode!=FILL_MODE_LINES) {
		setfillmode(FILL_MODE_LINES);
		draweditobj3d();
	}
	
	return 1;
}

int rawkey(int key, int qualifier, int doubleclick, struct windowdata * windat)
{
    switch (key) {
	case 1:			/* 1=draw edit points */
	draweditpoints();
	return 1;
    case 2:			/* 2=draw edit draw */
	draweditdraw();
    case 3:			/* 3=draw edit object */
	draweditobject();
	return 1;
    case 4:			/* 4=update display */
	updateeditdisplay();
	return 1;
    case 5:			/* 5=center to cursor */
	setcenter(&cursor);
	updateeditdisplay();
	return 1;
    case 0x10:			/* Q=quit */
	return 0;
    case 80:			/* F1=open x window */
	openwindow(&editxwindow);
	seteditmenustate(MENU_PREFS,MENU_PREFS_X,~0,1);
	updateeditdisplay();
	return 1;
    case 81:			/* F2=open y window */
	openwindow(&editywindow);
	seteditmenustate(MENU_PREFS,MENU_PREFS_Y,~0,1);
	updateeditdisplay();
	return 1;
    case 82:			/* F3=open z window */
	openwindow(&editzwindow);
	seteditmenustate(MENU_PREFS,MENU_PREFS_Z,~0,1);
	updateeditdisplay();
	return 1;
    case 85:			/* F6=drawwin to front */
	windowfront(DRAWWINDOW);
	return 1;
    case 86:			/* F7=drawwin to backdrop */
	windowback(DRAWWINDOW);
	return 1;
    case 88:			/* F9=NOP */
    case 89:			/* F10=NOP */
	break;
    case 64:			/* SPACE=select point */
	selectnearcursor(windat->type);
	return 1;
    case 69:			/* ESC=reset(deselect and release grab) */
	tool_cancel();
	return 1;
    case 62:			/* num 8=rot obj x */
	roteditobj(10, X_AXEL);
	draweditobj3d();
	return 1;
    case 30:			/* num 2=rot obj -x */
	roteditobj(-10, X_AXEL);
	draweditobj3d();
	return 1;
    case 45:			/* num 4=rot obj z */
	roteditobj(10, Z_AXEL);
	draweditobj3d();
	return 1;
    case 47:			/* num 6=rot obj -z */
	roteditobj(-10, Z_AXEL);
	draweditobj3d();
	return 1;
    case 61:			/* num 7=rot obj y */
    case 29:			/* num 1=rot obj y */
	roteditobj(10, Y_AXEL);
	draweditobj3d();
	return 1;
    case 63:			/* num 9=rot obj -y */
    case 31:			/* num 3=rot obj -y */
	roteditobj(-10, Y_AXEL);
	draweditobj3d();
	return 1;
    case 67:			/* Enter=reset rot obj */
	reseteditobj3d();
	return 1;
    case 46:			/* num 5=redraw object */
	draweditobj3d();
	return 1;
    case 0x14:			/* T */
	if(fillmode!=FILL_MODE_FILL) {
	    	seteditmenustate(MENU_PREFS,MENU_PREFS_FILL,~0,1);
		setfillmode(FILL_MODE_FILL);
		draweditobj3d();
	}
    	return 1;
    case 0x15:			/* Y */
	if(fillmode!=FILL_MODE_LINES) {
	    	seteditmenustate(MENU_PREFS,MENU_PREFS_LINES,~0,1);
		setfillmode(FILL_MODE_LINES);
		draweditobj3d();
	}
    	return 1;
    case 0x11:			/* W */
    case 0x12:			/* E */
    case 0x13:			/* R */
    case 0x16:			/* U */
    case 0x17:			/* I */
    case 0x18:			/* E */
    case 0x19:			/* O */
    case 0x20:			/* P */
    case 0x21:			/*  */
    case 0x22:			/*  */
    default:			/* xxxx=NOP */
	if (key & (1 << 7))
	    return 1;
	saydebug("KEY: %d\n", key);
	return 1;
    }
    return 1;
}

void newsize(struct windowdata * windat)
{
    updateeditdisplay();
}
