/*
 * source/graphics/graphics.c
 * SpacePigs VectorMania Graphics Support
 *
 * history: 1991-04-26 : New source: Graphics.c worked on init and basic
 * routines 1991-04-27 : worded on drawtypes
 */

#include <stdio.h>
#include <string.h>

#include <exec/types.h>
#include <exec/memory.h>
#include <utility/tagitem.h>
#include <graphics/gfx.h>
#include <graphics/gfxmacros.h>
#include <graphics/gfxbase.h>
#include <intuition/intuition.h>
#include <libraries/gadtools.h>

#include <amigasupport/closewindowsafely.h>

#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#include <proto/gadtools.h>

#include "other/debug.h"
#include "vector/vector.h"
#include "graphics/graphics.h"

extern char ProgId[];
extern char Version[];
extern int Revision;
extern char UserId[];
extern char Date[];

static struct Screen *screen = NULL;
struct IntuitionBase *IntuitionBase = NULL;
struct GfxBase *GfxBase = NULL;
static struct RastPort *rp;
static UWORD width, height;
static int maxx, minx;
static int maxy, miny;
static struct windowdata *currentwindow;
static struct MsgPort *WindowIDCMPPort;

/* AreaFill stuff */
static short areabuffer[AREA_SIZE];
static struct AreaInfo areainfo;
static struct TmpRas tmpras;
static PLANEPTR tmprasplane;
static struct RastPort cursor_rp;
static struct BitMap cursor_bm;
static struct RasInfo cursor_ri;
FILLMODE fillmode=FILL_MODE_FILL;

/* GadTools stuff */
struct Library *GadToolsBase = NULL;
APTR visualinfo = NULL;

char ScreenTitle[200];		/* Ifylles av GFXINIT och anvnds av NewScreen */
extern struct NewScreen NewScreen;
extern struct TagItem NewScreenTags[];
extern UWORD Palette[];
extern struct NewWindow NewDrawWindow;
extern struct windowdata drawwindow;

static long centx, centy;
static int currentcolor, currentocolor;
static int cursorcolor;

static UWORD chip cursor[] =
{
    0x0040, 0x0000, 0x0040, 0x0000, 0x0040, 0x0000, 0x0040, 0x0000, 0xFF1F, 0xE000,
    0x0040, 0x0000, 0x0040, 0x0000, 0x0040, 0x0000, 0x0040, 0x0000
};

void setfillmode(FILLMODE mode)
{
	fillmode=mode;
}
		
void movedelta(int x, int y)
{
    Move(rp, rp->cp_x + x, rp->cp_y + y);
}

void drawdelta(int x, int y)
{
    Move(rp, rp->cp_x + x, rp->cp_y + y);
}

void psetdelta(int x, int y)
{
    WritePixel(rp, rp->cp_x + x, rp->cp_y + y);
}

void pset(CORD3D * cord)
{
    WritePixel(rp, centx + cord->x, centy - IMULIF(cord->y, XYRATIO));
}

void draw(CORD3D * co)
{
    Draw(rp, centx + co->x, centy - IMULIF(co->y, XYRATIO));
}

void move(CORD3D * co)
{
    Move(rp, centx + co->x, centy - IMULIF(co->y, XYRATIO));
}

void areamove(CORD3D * co)
{
    AreaMove(rp, centx + co->x, centy - IMULIF(co->y, XYRATIO));
}

void areadraw(CORD3D * co)
{
    AreaDraw(rp, centx + co->x, centy - IMULIF(co->y, XYRATIO));
}

void areaelipse(CORD3D * cord, short xradie, short yradie)
{
    AreaEllipse(rp, centx + cord->x, centy - IMULIF(cord->y, XYRATIO)
		,xradie, yradie);
}

void areacircle(CORD3D * cord, short radie)
{
    int x, y, radiex, radiey;
    x = centx + cord->x;
    y = centy - IMULIF(cord->y, XYRATIO);
    radiex = radie;
    radiey = IMULIF(radie, XYRATIO);
    AreaEllipse(rp, x, y, radiex, radiey);
}

void areaend(void)
{
    AreaEnd(rp);
}

void circle(CORD3D * cord, int radie)
{
    int x, y, radiex, radiey;
    x = centx + cord->x;
    y = centy - IMULIF(cord->y, XYRATIO);
    radiex = radie;
    radiey = IMULIF(radie, XYRATIO);
    if (x + radiex < minx)
	return;
    if (x - radiex > maxx)
	return;
    if (y + radiey < miny)
	return;
    if (y - radiey > maxy)
	return;
    DrawEllipse(rp, x, y, radiex, radiey);
}

void line(CORD3D * cordbas, char start, char end)
{
    move(&cordbas[start]);
    draw(&cordbas[end]);
}

void surface(CORD3D * cordbas, char numcords, char *cords)
{
    int cord;
    if (numcords < 3)
	return;
    switch(fillmode) {
    case FILL_MODE_FILL:
    	areamove(&cordbas[cords[0]]);
    	for (cord = 1; cord < numcords; cord++) {
	    areadraw(&cordbas[cords[cord]]);
    	}
    	areaend();
    	break;
    case FILL_MODE_LINES:
    	move(&cordbas[cords[0]]);
    	for (cord = 1; cord < numcords; cord++) {
	    draw(&cordbas[cords[cord]]);
    	}
    	draw(&cordbas[cords[0]]);
    	break;
    }
}

void initgfx(struct MsgPort * IDCMPPort)
{
    WindowIDCMPPort = IDCMPPort;

    IntuitionBase = (struct IntuitionBase *)
	OpenLibrary("intuition.library", 33);
    if (IntuitionBase == NULL)
	ERROR("Can't open Intuition\n", 20);
    GfxBase = (struct GfxBase *) OpenLibrary("graphics.library", 33);
    if (GfxBase == NULL)
	ERROR("Can't open Graphics\n", 20);
    GadToolsBase = OpenLibrary("gadtools.library", 37);
    if (GadToolsBase == NULL)
	ERROR("Can't open gadtools.library", 20);

    sprintf(ScreenTitle, "%s %s_%02d %s by %s",
	    ProgId, Version, Revision, Date, UserId);
    screen = OpenScreenTagList(&NewScreen, NewScreenTags);
    if (screen == NULL)
	ERROR("Can't open screen\n", 20);
    LoadRGB4(&screen->ViewPort, Palette, 1 << NewScreen.Depth);
    visualinfo = GetVisualInfoA(screen, NULL);
    if (visualinfo == NULL)
	ERROR("Can't get VisualInfo on screen...\n", 20);
    initareafill();

    currentcolor = WHITE;
    currentocolor = GRAY4;

    if (!openwindow(&drawwindow))
	ERROR("Can't open drawwindow\n", 20);
    init_cursor();
    selectwindow(&drawwindow);
}


void closegfx(void)
{
    closeareafill();
    closewindow(&drawwindow);
    if (visualinfo)
	FreeVisualInfo(visualinfo);
    if (GadToolsBase)
	CloseLibrary(GadToolsBase);
    if (screen)
	CloseScreen(screen);
    if (IntuitionBase)
	CloseLibrary((struct Library *) IntuitionBase);
    if (GfxBase)
	CloseLibrary((struct Library *) GfxBase);
}

void clearscreen(void)
{
    SetRast(rp, 0);
    /* flush cursorbuffer and position.. */
    currentwindow->flags &= ~(WINFLAG_CURSORDISP | WINFLAG_CURSORSET);
}

void initareafill(void)
{
    int i;

    width = screen->Width;
    height = screen->Height;

    /* Init AraeFill buffer */
    for (i = 0; i < AREA_SIZE; i++)
	areabuffer[i] = 0;
    InitArea(&areainfo, areabuffer, (AREA_SIZE * 2) / 5);

    /* Init AraeFill plane */
    tmprasplane = AllocRaster(width, height);
    if (!tmprasplane)
	ERROR("Cant alloc tmpRaspPlane\n", 20);
    InitTmpRas(&tmpras, tmprasplane, RASSIZE(width, height));
}

void closeareafill(void)
{
    /* free areafill stuff */
    if (tmprasplane)
	FreeRaster(tmprasplane, width, height);
    tmprasplane = NULL;
}

struct Window *
 openwindow(struct windowdata * windat)
{
    struct Window *win;
    struct NewWindow *newwindow;
    PLANEPTR cursorbuffer;

    if (windat == DRAWWINDOW)
	windat = &drawwindow;

    if (windat->window) {
	saydebug("Window already opened:%d\n", windat->type);
	windowfront(windat);
	return windat->window;
    }
    newwindow = windat->newwindow;
    if (!newwindow) {
	saydebug("No newwindow???:%d\n", windat->type);
	return NULL;
    }
    newwindow->Screen = screen;

    if (!WindowIDCMPPort) {
	saydebug("openwindow:%d need an IDCMPPort if using IDCMP\n"
		 ,windat->type);
	newwindow->IDCMPFlags = NULL;
    }
    win = OpenWindowSafely(newwindow, WindowIDCMPPort);
    if (win == NULL) {
	saydebug("Can't open window:%d\n", windat->type);
	return NULL;
    }
    win->RPort->AreaInfo = &areainfo;
    win->RPort->TmpRas = &tmpras;

    makewindowdata(windat, win);

    cursorbuffer = (PLANEPTR) AllocMem(CURSORPLANESIZE * 4, MEMF_CHIP);
    if (cursorbuffer == NULL)
	saydebug("Can't alloc CursorBuffer\n", 0);
    windat->cursorbuffer = (char *) cursorbuffer;

    selectwindow(windat);

    SetDrMd(rp, JAM1);
    return win;
}

void closewindow(struct windowdata * windat)
{
    struct Window *win;

    if (windat == DRAWWINDOW)
	windat = &drawwindow;

    win = windat->window;
    if (!win)
	return;

    if (windat->flags & WINFLAG_MENUON)
	menu_off(windat);
    CloseWindowSafely(win);
    if (windat->cursorbuffer) {
	FreeMem(windat->cursorbuffer, CURSORPLANESIZE * 4);
    }
    windat->cursorbuffer = NULL;
    windat->window = NULL;
    windat->rastport = NULL;
}

int selectwindow(struct windowdata * windat)
{
    if (windat == DRAWWINDOW)
	windat = &drawwindow;
    if (!windat->window) {
	saydebug("Tries to select closed window:%d\n", windat->type);
	if (windat->type == WINDOW_DRAW)
	    ERROR("Drawwindow CAN'T be closed, it MUST be open.\n"
		  ,20);
	return 0;
    }
    currentwindow = windat;
    rp = windat->rastport;
    centx = windat->xcenter;
    centy = windat->ycenter;
    minx = windat->minx;
    miny = windat->miny;
    maxx = windat->maxx;
    maxy = windat->maxy;
    setcolor(currentcolor);
    setocolor(currentocolor);
    return 1;
}

void makewindowdata(struct windowdata * windat, struct Window * win)
{
    if (windat == DRAWWINDOW)
	windat = &drawwindow;
    windat->window = win;
    windat->rastport = win->RPort;
    getwindowdata(windat);
    win->UserData = (APTR) windat;
}

void getwindowdata(struct windowdata * windat)
{
    int centx, centy;
    int width, height;
    int miny, maxy;
    int minx, maxx;
    struct Window *win;

    if (windat == DRAWWINDOW)
	windat = &drawwindow;
    if (!windat)
	ERROR("getwindowdata MUST have a windowdata!!", 20);

    win = windat->window;
    if (!win) {
	saydebug("getwindowdata no window??:%d\n", windat->type);
	ERROR("might be dangerous.. quiting now", 20);
    }
    if (win->Flags & GIMMEZEROZERO) {
	width = win->GZZWidth;
	height = win->GZZHeight;
	miny = 0;
	minx = 0;
	maxy = height - 1;
	maxx = width - 1;
	centx = width / 2;
	centy = height / 2;
    } else {
	width = win->Width;
	height = win->Height;
	minx = win->BorderLeft;
	miny = win->BorderTop;
	maxx = width - win->BorderRight - 1;
	maxy = height - win->BorderBottom - 1;
	centx = (maxx - minx) / 2 + minx;
	centy = (maxy - miny) / 2 + miny;
    }
    windat->xcenter = centx;
    windat->ycenter = centy;
    windat->minx = minx;
    windat->maxx = maxx;
    windat->miny = miny;
    windat->maxy = maxy;

    if (windat == currentwindow)
	selectwindow(windat);
}

int setcolor(int color)
{
    int oldcolor;
    oldcolor = currentcolor;
    if (rp)
	SetAPen(rp, color);
    currentcolor = color;
    return oldcolor;
}

int setocolor(int color)
{
    int oldcolor;
    oldcolor = currentocolor;
    if (rp)
	SetOPen(rp, color);
    currentocolor = color;
    return oldcolor;
}

void setgadget(struct windowdata * windat, int gadgetid, int state)
{
    USHORT flags;

    struct Window *win;
    struct Gadget *gadget;

    if (windat == DRAWWINDOW)
	windat = &drawwindow;
    win = windat->window;
    if(win) {
	gadget = win->FirstGadget;
    } else {
    	gadget = windat->newwindow->FirstGadget;
    }
	    
    while (gadget->GadgetID != gadgetid) {
	gadget = gadget->NextGadget;
	if (gadget == NULL) {
	    printf("Can't find gadget %d in window %d\n", gadgetid, windat->type);
	    return;
	}
    }
    if(win) {
	RemoveGadget(win, gadget);
    }
    flags = gadget->Flags;
    flags &= ~SELECTED;		/* Clear selected flag */
    if (state)
	flags |= SELECTED;	/* set selected flag if selected */
    gadget->Flags = flags;
    if(win) {
    	AddGadget(win, gadget, ~0);
    	RefreshGList(gadget, win, NULL, 1);
    }
}

void windowfront(struct windowdata * windat)
{
    struct Window *win;
    if (windat == DRAWWINDOW)
	windat = &drawwindow;
    win = windat->window;

    if (win == NULL) {
	saydebug("Window to front, window %d is closed??\n"
		 ,windat->type);
	return;
    }
    if (win->Flags & BACKDROP) {
	long oldflags;
	closewindow(windat);
	oldflags = windat->newwindow->Flags;
	windat->newwindow->Flags &= ~BACKDROP;
	openwindow(windat);
	windat->newwindow->Flags = oldflags;
	windowfront(windat);
	return;
    }
    WindowToFront(win);
}

void windowback(struct windowdata * windat)
{
    struct Window *win;
    if (windat == DRAWWINDOW)
	windat = &drawwindow;
    win = windat->window;

    if (win == NULL) {
	saydebug("Window to back, window %d is closed??\n"
		 ,windat->type);
	return;
    }
    if (windat->newwindow->Flags & BACKDROP) {
	if (!(win->Flags & BACKDROP)) {
	    long oldflags;
	    closewindow(windat);
	    oldflags = windat->newwindow->Flags;
	    windat->newwindow->Flags |= BACKDROP;
	    openwindow(windat);
	    windat->newwindow->Flags = oldflags;
	    windowback(windat);
	    return;
	}
    }
    WindowToBack(win);
}

void draw_ball(CORD3D * cord, short radie)
{
	switch(fillmode) {
	case FILL_MODE_FILL:
	    areacircle(cord, radie);
	    areaend();
	    break;
	case FILL_MODE_LINES:
	    circle(cord,radie);
	    break;
	}
}

void init_cursor(void)
{
    InitBitMap(&cursor_bm, 4, CURSORWIDTH, CURSORHEIGHT);
    InitRastPort(&cursor_rp);
    cursor_rp.BitMap = &cursor_bm;
    setcursorcolor(WHITE);
}

int setcursorcolor(int color)
{
    int oldcol;
    oldcol = cursorcolor;
    cursorcolor = color;
    return oldcol;
}

void setcursor(CORD3D * pos)
{
    int x, y;
    setcursorbm();
    restore_cursor();
    x = centx + pos->x - CURSORCENTX;
    y = centy - IMULIF(pos->y, XYRATIO) - CURSORCENTY;
    currentwindow->cursorx = x;
    currentwindow->cursory = y;
    currentwindow->flags |= WINFLAG_CURSORSET;
    if (currentwindow->flags & WINFLAG_CURSORON)
	draw_cursor(x, y);
}

void restore_cursor(void)
{
    if (!(currentwindow->flags & WINFLAG_CURSORDISP))
	return;
    ClipBlit(&cursor_rp, 0, 0, rp, currentwindow->cursorx
	     ,currentwindow->cursory, CURSORWIDTH, CURSORHEIGHT, 0xc0);
    currentwindow->flags &= ~WINFLAG_CURSORDISP;
}

void draw_cursor(short x, short y)
{
    if (currentwindow->flags & WINFLAG_CURSORSET) {
	/* Save screendata under cursor */
	ClipBlit(rp, x, y, &cursor_rp, 0, 0, CURSORWIDTH, CURSORHEIGHT, 0xc0);
	/* Display cursor */
	SetAPen(rp, cursorcolor);
	BltTemplate((PLANEPTR) & cursor, 0, CURSORBWIDTH, rp, x, y
		    ,CURSORWIDTH, CURSORHEIGHT);
	SetAPen(rp, currentcolor);
	currentwindow->flags |= WINFLAG_CURSORDISP;
    }
}

void cursoroff(void)
{
    currentwindow->flags &= ~WINFLAG_CURSORON;
    if (currentwindow->flags & WINFLAG_CURSORDISP) {
	setcursorbm();
	restore_cursor();
    }
}

void cursoron(void)
{
    if (currentwindow->cursorbuffer == NULL) {
	saydebug("cursonon:%d No cursor buffer?\n", currentwindow->type);
	return;
    }
    currentwindow->flags |= WINFLAG_CURSORON;
    setcursorbm();
    draw_cursor(currentwindow->cursorx, currentwindow->cursory);
}

void setcursorbm(void)
{
    short plane;
    PLANEPTR bufplane;
    bufplane = currentwindow->cursorbuffer;

    for (plane = 0; plane < 4; plane++, bufplane += CURSORPLANESIZE) {
	cursor_bm.Planes[plane] = bufplane;
    }
}

void menu_on(struct windowdata * windat)
{
    struct menudata *menudat;

    if (!windat->window) {
	saydebug("menu_on: Window NOT open:%d\n", windat->type);
	return;
    }
    if (windat->flags & WINFLAG_MENUON) {
	saydebug("menu_on twice on this window...:%d\n", windat->type);
	return;
    }
    menudat = windat->menudat;
    if (menudat == NULL) {
	saydebug("menu_on without a menu??:%d\n", windat->type);
	return;
    }
    /* Menyn fr endast vara aktiv i ett fnster i taget.. */
    if (menudat->currentwin) {
	saydebug("menu_on twice on this menu..\n", 0);
	menu_off(menudat->currentwin);
    }
    if (menudat->flags & MENUFLAG_UPDATED) {
	SetMenuStrip(windat->window, menudat->menu);
	menudat->flags &= ~MENUFLAG_UPDATED;
    } else {
	ResetMenuStrip(windat->window, menudat->menu);
    }
    windat->flags |= WINFLAG_MENUON;
    /* Tala om i vilket fnster menyn r aktiv nu.. */
    menudat->currentwin = windat;
}

void menu_off(struct windowdata * windat)
{
    if (!(windat->flags & WINFLAG_MENUON)) {
	saydebug("menu already off:%d\n", windat->type);
	return;
    }
    if (!windat->window) {
	saydebug("menu_off: Window NOT open:%d\n", windat->type);
	return;
    }
    ClearMenuStrip(windat->window);
    windat->flags &= ~WINFLAG_MENUON;

    /* Tala om att menyn inte r aktiv lngre */
    windat->menudat->currentwin = NULL;
}

void setmenu(struct windowdata * windat, struct menudata * menudat)
{
    int menustate;
    menustate = (windat->flags & WINFLAG_MENUON);
    if (menustate) {
	menu_off(windat);
    }
    windat->menudat = menudat;
    if (menustate) {
	menu_on(windat);
    } else {
	saydebug("setmenu: no menu?:%d\n", windat->type);
	return;
    }
}

void createmenu(struct menudata * menudat)
{
    struct Menu *menu;

    menu = CreateMenusA(menudat->newmenu, menudat->newmenutags);
    if (menu == NULL)
	ERROR("Can't create menu\n", 20);
    menudat->menu = menu;
    menudat->flags |= MENUFLAG_UPDATED;
    if (!LayoutMenusA(menu, visualinfo, NULL))
	ERROR("Can't layout menu\n", 20);
}

void freemenu(struct menudata * menudat)
{
    struct Menu *menu;
    menu = menudat->menu;
    if (menu)
	FreeMenus(menu);
}

void set_menustate(struct menudata * menudat, int menu, int item, int sub, int state)
{
    UWORD menunum;
    struct windowdata *windat;
    struct Window *win;

    windat = menudat->currentwin;
    if (!windat) {
	saydebug("set_menustate: needs active window...\n", 0);
	return;
    }
    win = windat->window;
    if (!win) {
	saydebug("set_menustate: needs open window...\n", 0);
	return;
    }
    menunum = FULLMENUNUM(menu, item, sub);
    if (state)
	OnMenu(win, menunum);
    else
	OffMenu(win, menunum);
}

void setmenustate(struct menudata * menudat, int menu, int item, int sub, int state)
{
    struct MenuItem *menuitem;
    long exclude;
    int i;
    
    menu&=NOMENU;
    item&=NOITEM;
    sub&=NOSUB;
    
    if (menudat->currentwin) {
	ClearMenuStrip(menudat->currentwin->window);
    }
    menuitem = ItemAddress(menudat->menu, FULLMENUNUM(menu, item, sub));
    if (menuitem) {
	menuitem->Flags &= ~CHECKED;
	if (state) {
	    menuitem->Flags |= CHECKED;
	}
	exclude = menuitem->MutualExclude;
	if (exclude) {
	    i=0;
	    if(sub!=NOSUB) {
	    	menuitem = ItemAddress(menudat->menu,FULLMENUNUM(menu,item,0));
	    } else {
	    	menuitem = ItemAddress(menudat->menu,FULLMENUNUM(menu,0,NOSUB));
	    }
	    while(menuitem && i<NOITEM) {
		if (exclude & (1 << i)) {
		    if (menuitem->Flags & CHECKIT) {
			menuitem->Flags &= ~CHECKED;
		    }
		}
		menuitem=menuitem->NextItem;
		i+=1;
	    }
	}
    } else {
	printf("setmenustate: can't find menuitem %08x\n", FULLMENUNUM(menu, item, sub));
    }

    if (menudat->currentwin) {
	ResetMenuStrip(menudat->currentwin->window, menudat->menu);
    }
}

int getmenustate(struct menudata * menudat, int menu, int item, int sub)
{
    struct MenuItem *menuitem;
    menuitem = ItemAddress(menudat->menu, FULLMENUNUM(menu, item, sub));
    if(!menuitem) {
    	saydebug("getmenustate: unknown menu item:%d\n",item);
    	return 0;
    }
    return menuitem->Flags&CHECKED;
}
