/*
 * source/vector/vectmath.h
 * SpacePigs VectorMania Vector math structurer, macros och prototyper
 *
 * history:
 *  1991-04-15: Math work begun
 *  1991-04-20: Basic thinking
 *  1991-04-27: Added prototypes
 */

#define PI   3.14159265358979323846
#define VDIST			(1000)
#define PERSPECTIVEZOFFSET	(2*VDIST)
#define PERSPECTIVEZFACTOR	(-VDIST*VDIST)

#define FIXEDPOINTMATH
typedef long VECTFLOAT;
typedef short VECTINT;

#define FIXEDBITNUM	(8)
#define FIXEDUNIT	(1<<FIXEDBITNUM)
#define VECTFLOATUNIT	FIXEDUNIT

/* VECTFLOAT to float (double) */
#define FTOFL(x)	((x)/(double)FIXEDUNIT)
/* float (double) to VECTFLOAT */
#define FLTOF(x)	((VECTFLOAT)((x)*(double)FIXEDUNIT))
/* VECTFLOAT to VECTINT */
#define FTOI(x)	((x)>>FIXEDBITNUM)
/* SIN to VECTFLOAT */
#define STOF(x) ((x)>>(15-FIXEDBITNUM))
/* SIN to VECTFLOAT */
#define STOI(x) ((x)>>15)
/* VECTINT to VECTFLOAT */
#define ITOF(x)	((VECTFLOAT)(x)<<FIXEDBITNUM)
/* Multiply VECTINT*VECTFLOAT=VECTFLOAT */
#define FMULIF(i,f)	(i*f)
/* Multiply VECTINT*VECTFLOAT=VECTINT */
#define IMULIF(i,f)	(FTOI(FMULIF(i,f)))
/* Multiply VECTINT*SIN=VECTINT */
#define IMULIS(i,s)	(((i)*(s))>>15)
/* Multiply VECTINT*SIN=VECTFLOAT */
#define FMULIS(i,s)	(((i)*(s))>>(15-FIXEDBITNUM))
/* Multiply VECTINT*VECTINT */
#define MULII(a,b)	(a*b)
/* Multiply VECTFLOAT*VECTFLOAT */
#define MULFF(a,b)	(a*b/FIXEDUNIT)
/* VECTFLOAT divide adjust */
#define FDIVADJ(x)	((x)*FIXEDUNIT)
/* Divide VECTINT/VECTFLOAT=VECTFLOAT */
#define FDIVIF(i,f)	(FDIVADJ(ITOF(i))/(f))
/* Divide VECTFLOAT/VECTINT=VECTFLOAT */
#define FDIVFI(f,i)	((f)/(i))
/* Divide VECTINT/VECTFLOAT=VECTINT */
#define IDIVIF(i,f)	(FDIVADJ(i)/(f))
/* Divide VECTFLOAT/VECTINT=VECTINT */
#define IDIVFI(i,f)	(FTOI((f)/(i)))
/* Divide VECTINT/VECTINT */
#define DIVII(a,b)	(a/b)
/* Divide VECTFLOAT/VECTFLOAT */
#define DIVFF(a,b)	(((a)*FIXEDUNIT)/(b))
/* rounded Divide VECTINT/VECTFLOAT=VECTFLOAT */
#define RFDIVIF(i,f)	((FDIVADJ(ITOF(i))+((f)>>1))/(f))
/* rounded Divide VECTFLOAT/VECTINT=VECTFLOAT */
#define RFDIVFI(f,i)	((f+((i)>>1))/(i))
/* rounded Divide VECTINT/VECTFLOAT=VECTINT */
#define RIDIVIF(i,f)	((FDIVADJ(i)+((f)>>1))/(f))
/* rounded Divide VECTFLOAT/VECTINT=VECTINT */
#define RIDIVFI(i,f)	(FTOI(((f)+((i)>>1))/(i)))
/* rounded Divide VECTINT/VECTINT */
#define RDIVII(a,b)	(((a)+((b)>>1))/(b))
/* rounded Divide VECTFLOAT/VECTFLOAT */
#define RDIVFF(a,b)	((FDIVADJ(a)+((b)>>1))/(b))

#define ABS(val)	((val)<0?-(val):(val))

typedef struct Cord3d {
    VECTINT x;
    VECTINT y;
    VECTINT z;
    VECTINT normz;
} CORD3D;

#define SETCORD2D(st,xc,yc)	{st.x=xc;st.y=yc;}
#define SETCORD3D(st,xc,yc,zc)	{st.x=xc;st.y=yc;st.z=zc;}

typedef enum Axel {
    X_AXEL, Y_AXEL, Z_AXEL
} AXEL;

/* Vector math internal routines */
void rotate(VECTINT x, VECTINT y, VECTINT * x1, VECTINT * y1, short si, short co);
void rotx(CORD3D * source, CORD3D * dest, int angle);
void roty(CORD3D * source, CORD3D * dest, int angle);
void rotz(CORD3D * source, CORD3D * dest, int angle);
void perspective(CORD3D * source, struct Cord2d * dest);
void rotxc(CORD3D * source, CORD3D * dest, short si, short co);
void rotyc(CORD3D * source, CORD3D * dest, short si, short co);
void rotzc(CORD3D * source, CORD3D * dest, short si, short co);
int getsincos(int angle, short *si, short *co);

/* Vector math support prototypes */
void rotcords(CORD3D * source, CORD3D * dest, int antal, AXEL axel, int angle);
void perspectivecords(CORD3D * source, CORD3D * dest, int antal);
void copycords(CORD3D * source, CORD3D * dest, int antal);
void scalecords(CORD3D * source, CORD3D * dest, int antal, VECTFLOAT scale);
void stretchcords(CORD3D * source, CORD3D * dest, int antal, CORD3D * scale);
void transcords(CORD3D * source, CORD3D * dest, int antal, CORD3D * trans);
void flipcords(CORD3D * source, CORD3D * dest, int antal, AXEL axel);
int cmpcords(CORD3D * c1, CORD3D * c2);
