/*
 * source/Vector/Vector_Generellt.c
 * SpacePigs VectorMania general vector routines
 */
#include <stdlib.h>

#include "other/debug.h"
#include "vector/vector.h"
#include "graphics/graphics.h"

ELEMENT *elements[MAXELEMS];
int numelems;

CORD3D *cbase;

int elements_num = 0;

void setcordbas(CORD3D * cbas)
{
    cbase = cbas;
}

void init_elem(ELEMENT *elem, STDDRAWTYPE *elemdraw)
{
    elem->type = elemdraw->type;
    elem->cords = elemdraw->cords;
    elem->numcords = elemdraw->numcords;
    elem->color = elemdraw->color;
    elem->flags = 0;

    elem_maxmin(elem);
}


void elem_maxmin(ELEMENT *elem)
{
    CORD3D *max, *min;
    char cord, numcords;
    char *cords;

    max = &elem->max;
    min = &elem->min;
    numcords = elem->numcords;

    cords = elem->cords;
    *min = *max = cbase[cords[0]];

    for (cord = 0; cord < numcords; cord++) {
	CORD3D *this = &cbase[cords[cord]];

	max->x = MAX(max->x, this->x);
	max->y = MAX(max->y, this->y);
	max->z = MAX(max->z, this->z);
	min->x = MIN(min->x, this->x);
	min->y = MIN(min->y, this->y);
	min->z = MIN(min->z, this->z);
    }
}

ELEMENT *
 get_elem(enum draw_type type)
{
    ELEMENT *elem;
    int size;

    switch (type) {
    case DT_BALL:
	size = sizeof(ELEMENT_BALL);
	break;
    case DT_LINE:
	size = sizeof(ELEMENT_LINE);
	break;
    case DT_SURF:
	size = sizeof(ELEMENT_SURF);
	break;
    default:
	saydebug("Getelem:%d", type);
	ERROR(" unknown type\n", 10);
    }
    elem = malloc(size);
    if (elem == 0)
	ERROR("Cant alloc element!\n", 10);
    elem->elemnum = -1;
    elem->type = type;
    return elem;
}

void free_elem(ELEMENT * elem)
{
    if (elem == 0) {
	saydebug("Cant free NULL element!\n", 0);
	return;
    }
    if (elem->elemnum >= 0) {
	saydebug("Cant free elem, thats in elemlist:%d (Maybe i should remove it...)\n", elem->elemnum);
	return;
    }
    free(elem);
}

void makeelems(char *drawlist)
{
    STDDRAWTYPE *elemdraw=(STDDRAWTYPE *)drawlist;
    
    while (elemdraw) {
	elemdraw = makeelem(elemdraw);
    }
}

STDDRAWTYPE *
 makeelem(STDDRAWTYPE *elemdraw)
{
    ELEMENT *elem;
    switch (elemdraw->type) {
    case DT_END:
	return 0;		/* End of list */
    case DT_BALL:
	elem=makeelem_ball(elemdraw);
	break;
    case DT_LINE:
	elem=makeelem_line(elemdraw);
	break;
    case DT_SURF:
	elem=makeelem_surf(elemdraw);
	break;
    case DT_TOOLSTART:		/* makeelem skall ven ta med tool element */
    	return NEXTDRAW(elemdraw);
    default:
	saydebug("Cant make unknown drawtype:%d\n", elemdraw->type);
	return 0;		/* force end */
    }
    if(!elem) {
    	saydebug("makeelem: failed:%d\n",elemdraw->type);
    	return 0;
    }
    add_elem(elem);
    return NEXTDRAW(elemdraw);
}

ELEMENT *
 makeelem_ball(STDDRAWTYPE *elemdraw)
{
    ELEMENT_BALL *ball;
    BALLDRAWDATA *data;
    
    if(elemdraw->numcords!=1) {
    	saydebug("makeelem_ball: not 1 cord:%d\n",elemdraw->numcords);
    	return 0L;
    }
    data = (BALLDRAWDATA *)GETDRAWDATA(elemdraw);
    ball = (ELEMENT_BALL *) get_elem(DT_BALL);
    init_elem((ELEMENT *) ball, elemdraw);
    ball->radie = data->radie;
    return (ELEMENT *)ball;
}

ELEMENT *
 makeelem_line(STDDRAWTYPE *elemdraw)
{
    ELEMENT_LINE *line;
    
    if(elemdraw->numcords!=2) {
    	saydebug("makeelem_line: not 2 cords:%d\n",elemdraw->numcords);
    	return 0;
    }
    line = (ELEMENT_LINE *) get_elem(DT_LINE);
    init_elem((ELEMENT *) line, elemdraw);
    return (ELEMENT *)line;
}

ELEMENT *
 makeelem_surf(STDDRAWTYPE *elemdraw)
{
    ELEMENT_SURF *surf;
    
    if(elemdraw->numcords<3) {
    	saydebug("makeelem_surf: less than 3 cords:%d\n",elemdraw->numcords);
	return 0;
    }
    surf = (ELEMENT_SURF *) get_elem(DT_SURF);
    init_elem((ELEMENT *) surf, elemdraw);
    return (ELEMENT *)surf;
}

void add_elem(ELEMENT * elem)
{
    int i;
    int j;
    VECTINT zmin;
    VECTINT zmax;
    
    /* kontrollera att elemlistan inte blir verfull! */
    if (numelems >= MAXELEMS) {
	saydebug("ADD_ELEM: Elemlist full! free(elem)\n", 0);
	free_elem(elem);
	return;
    }
    /* sorteras efter zmin */
    zmin = ELEM_MIN(elem, z);
    for (i = 0; i < numelems && zmin < ELEM_MIN(elements[i], z); i++);
    /* samma zmin sorteras efter zmax */
    zmax = ELEM_MAX(elem,z);
    for (;i<numelems && zmin==ELEM_MIN(elements[i],z) && zmax < ELEM_MAX(elements[i],z) ;i++);
    /* gr plats fr elememt */
    for (j = numelems; j > i; j--) {
	elements[j] = elements[j - 1];
	elements[j]->elemnum = j;
    }
    /* Stt in element */
    elements[i] = elem;
    elem->elemnum = i;
    numelems += 1;
}

void exch_elems(ELEMENT *elem1, ELEMENT *elem2)
{
    int num1, num2;
    num1 = elem1->elemnum;	/* Find elemnums */
    num2 = elem2->elemnum;
    elem1->elemnum = num2;	/* Swap nums */
    elem2->elemnum = num1;
    elements[num1] = elem2;	/* Swap positions */
    elements[num2] = elem1;
}

void free_elements(void)
{
    ELEMENT **elem;
    int i;

    for (elem = elements, i = 0; i < numelems; i++, elem++) {
	(*elem)->elemnum = -1;
	free_elem(*elem);
	*elem = 0;
    }
    numelems = 0;
}

void draw_elements(void)
{
    ELEMENT **elem;
    int i;

    for (elem = elements, i = 0; i < numelems; i++, elem++) {
	draw_elem(*elem);
    }
}

void draw_elem(ELEMENT * elem)
{
    setcolor(elem->color);
    switch (elem->type) {
    case DT_BALL:
	drawelem_ball((ELEMENT_BALL *) elem);
	break;
    case DT_LINE:
	drawelem_line((ELEMENT_LINE *) elem);
	break;
    case DT_SURF:
	drawelem_surf((ELEMENT_SURF *) elem);
	break;
    default:
	saydebug("draw_elem:%d unknown drawtype\n", elem->type);
	break;
    }
}

void drawelem_ball(ELEMENT_BALL * ball)
{
    extern short per_vdist;
    CORD3D *cord;
    short radie;
    VECTINT z;

    cord = &cbase[ball->elem.cords[0]];

    z = cord->normz;
    radie = ball->radie;
    radie = (per_vdist * radie) / z;

    draw_ball(cord, radie);
}

void drawelem_line(ELEMENT_LINE * elem)
{
    line(cbase, elem->elem.cords[0], elem->elem.cords[1]);
}

void drawelem_surf(ELEMENT_SURF * surf)
{
    surface(cbase, surf->elem.numcords, surf->elem.cords);
}
