/*
 * source/vector/vector.h
 * SpacePigs VectorMania Vector support structures and macros
 */

#include "vector/vectmath.h"

#define MAX(a,b)	((a)>(b) ? a:b)
#define MIN(a,b)	((a)<(b) ? a:b)

#define ELEM_MAX(elem,cord)	(elem->max.cord)
#define ELEM_MIN(elem,cord)	(elem->min.cord)

#ifndef NULL
#define NULL 0L
#endif

#define MAXELEMS 1000

/* DRAW TYPE
**  FORMAT:
**   TYPE,SIZE,COLOR,NUMCORDS,CORDS[],DATA
**
**  TYPER:
**   DT_END		END OF DRAW, ENBART TYP
**   DT_BALL		BOLL	ENDAST 1 kordinat
**			DATA:	RADIE=bollens radie
**   DT_LINE		LINJE	ENDAST 2 kordinater
**   DT_SURF		YTA	X antal kordinate
**   DT_SURFFRONT	YTA	X antal kordinater, tas bort om baksida
**   DT_TOOLSTART	START OF TOOL DRAWLIST
*/
typedef enum draw_type {
    DT_TOOLSTART = -1, DT_END = 0, DT_BALL, DT_LINE, DT_SURF, DT_SURFFRONT
} DRAWTYPE;

#define GETDRAWDATA(d) ((STDDRAWDATA *)(((char *)d)+CALCDRAWSIZE(d->numcords,0)))
#define NEXTDRAW(d) ((STDDRAWTYPE *)(((char *)d)+d->size))
#define CALCDRAWSIZE(a,b) (sizeof(STDDRAWTYPE)-1+a+b)

typedef struct stddrawtype {
    unsigned char type;			/* enum draw_type values */
    unsigned char size;			/* total storlek */
    unsigned char color;		/* frg */
    unsigned char numcords;		/* antal kordinater  */
    unsigned char cords[1];		/* [numcords] kordinatnummer */
} STDDRAWTYPE;

typedef void STDDRAWDATA;

typedef struct balldrawdata {
    unsigned char radie;
} BALLDRAWDATA;

#define ELEMF_SORTED (1<<0)

typedef struct element {
    char type;			/* vad r detta(DT_LINE osv) */
    char flags;			/* ELEM_MOVED om redan sorterad en gng */
    int elemnum;		/* Elementets nummer i elementlistan */
    char color;			/* Elementets frg */
    char numcords;		/* Hur mnga kordinater har detta element ? */
    char *cords;		/* Kordinatnummer */
    CORD3D max, min;	/* den omskrivna rektangeln */
} ELEMENT;

typedef struct element_ball {
    ELEMENT elem;
    char radie;
    /* Bollens radie */
} ELEMENT_BALL;

typedef struct element_line {
    ELEMENT elem;
} ELEMENT_LINE;

#define ELEMF_SURFEQU

typedef struct element_surf {
    ELEMENT elem;
    VECTFLOAT A,B,C,D;		/* Ifyllta om ELEMF_SURFEQU satt */
} ELEMENT_SURF;

/* Generella funktioner */
void setcordbas(CORD3D * cbas);	/* Stt kordinatbas fr elementen */
void init_elem(ELEMENT *elem, STDDRAWTYPE *draw);	/* Initiera ett element */
void elem_maxmin(ELEMENT * elem);	/* berkna elementets MAX och MIN */
ELEMENT *get_elem(enum draw_type type);	/* Allocera ett element */
void free_elem(ELEMENT * elem);	/* Deallocera ett element */
void free_elements(void);	/* Deallocera alla element i elementlistan */
void add_elem(ELEMENT * elem);	/* Stt in ett element i elementlistan */
void exch_elems(ELEMENT * elem1, ELEMENT * elem2);	/* Byt plats p tv element i elementlistan */
void makeelems(char *drawlist);	/* Gr alla element i en drawlista */
STDDRAWTYPE *makeelem(STDDRAWTYPE *draw);	/* Gr ett element ur drawlista */
ELEMENT *makeelem_ball(STDDRAWTYPE *draw);	/* Gr ett bollelement ur drawlista */
void draw_elements(void);	/* Rita alla element i elementlistan */
void draw_elem(ELEMENT * elem);	/* Rita ett element */
void drawelem_ball(ELEMENT * ball);	/* Rita ett boll element */
ELEMENT *makeelem_line(STDDRAWTYPE *draw);	/* Gr ett linjeelement ur drawlista */
ELEMENT *makeelem_surf(STDDRAWTYPE *draw);	/* Gr ett surfacelement ur drawlista */
void drawelem_line(ELEMENT * line);	/* Rita ett linje element */
void drawelem_surf(ELEMENT * surf);	/* Rita ett surface element */
